@@ -1,22 +1,21 @@
 # sybase/mxodbc.py
-# Copyright (C) 2005-2020 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2023 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
-# the MIT License: http://www.opensource.org/licenses/mit-license.php
+# the MIT License: https://www.opensource.org/licenses/mit-license.php
 """
 
 .. dialect:: sybase+mxodbc
     :name: mxODBC
     :dbapi: mxodbc
     :connectstring: sybase+mxodbc://<username>:<password>@<dsnname>
-    :url: http://www.egenix.com/
+    :url: https://www.egenix.com/
 
 .. note::
 
     This dialect is a stub only and is likely non functional at this time.
 
-
 """
 from sqlalchemy.connectors.mxodbc import MxODBCConnector
 from sqlalchemy.dialects.sybase.base import SybaseDialect
@@ -29,6 +28,7 @@ class SybaseExecutionContext_mxodbc(SybaseExecutionContext):
 
 class SybaseDialect_mxodbc(MxODBCConnector, SybaseDialect):
     execution_ctx_cls = SybaseExecutionContext_mxodbc
+    supports_statement_cache = True
 
 
 dialect = SybaseDialect_mxodbc
