@@ -1,13 +1,16 @@
 # ext/asyncio/session.py
-# Copyright (C) 2020-2022 the SQLAlchemy authors and contributors
+# Copyright (C) 2020-2023 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: https://www.opensource.org/licenses/mit-license.php
+import asyncio
+
 from . import engine
 from . import result as _result
 from .base import ReversibleProxy
 from .base import StartableContext
+from .result import _ensure_sync_result
 from ... import util
 from ...orm import object_session
 from ...orm import Session
@@ -208,7 +211,7 @@ class AsyncSession(ReversibleProxy):
         else:
             execution_options = _EXECUTE_OPTIONS
 
-        return await greenlet_spawn(
+        result = await greenlet_spawn(
             self.sync_session.execute,
             statement,
             params=params,
@@ -216,6 +219,7 @@ class AsyncSession(ReversibleProxy):
             bind_arguments=bind_arguments,
             **kw
         )
+        return await _ensure_sync_result(result, self.execute)
 
     async def scalar(
         self,
@@ -310,7 +314,9 @@ class AsyncSession(ReversibleProxy):
         **kw
     ):
         """Execute a statement and return a streaming
-        :class:`_asyncio.AsyncResult` object."""
+        :class:`_asyncio.AsyncResult` object.
+
+        """
 
         if execution_options:
             execution_options = util.immutabledict(execution_options).union(
@@ -501,7 +507,7 @@ class AsyncSession(ReversibleProxy):
         blocking-style code, which will be translated to implicitly async calls
         at the point of invoking IO on the database drivers.
 
-        """  # noqa E501
+        """  # noqa: E501
 
         return self.sync_session.get_bind(
             mapper=mapper, clause=clause, bind=bind, **kw
@@ -514,8 +520,8 @@ class AsyncSession(ReversibleProxy):
         This method may also be used to establish execution options for the
         database connection used by the current transaction.
 
-        .. versionadded:: 1.4.24  Added **kw arguments which are passed through
-           to the underlying :meth:`_orm.Session.connection` method.
+        .. versionadded:: 1.4.24  Added \**kw arguments which are passed
+           through to the underlying :meth:`_orm.Session.connection` method.
 
         .. seealso::
 
@@ -603,7 +609,7 @@ class AsyncSession(ReversibleProxy):
             :meth:`_asyncio.AsyncSession.close`
 
         """
-        return await greenlet_spawn(self.sync_session.close)
+        await greenlet_spawn(self.sync_session.close)
 
     async def invalidate(self):
         """Close this Session, using connection invalidation.
@@ -621,7 +627,8 @@ class AsyncSession(ReversibleProxy):
         return self
 
     async def __aexit__(self, type_, value, traceback):
-        await self.close()
+        task = asyncio.get_event_loop().create_task(self.close())
+        await asyncio.shield(task)
 
     def _maker_context_manager(self):
         # no @contextlib.asynccontextmanager until python3.7, gr
@@ -638,8 +645,12 @@ class _AsyncSessionContextManager:
         return self.async_session
 
     async def __aexit__(self, type_, value, traceback):
-        await self.trans.__aexit__(type_, value, traceback)
-        await self.async_session.__aexit__(type_, value, traceback)
+        async def go():
+            await self.trans.__aexit__(type_, value, traceback)
+            await self.async_session.__aexit__(type_, value, traceback)
+
+        task = asyncio.get_event_loop().create_task(go())
+        await asyncio.shield(task)
 
 
 class AsyncSessionTransaction(ReversibleProxy, StartableContext):
