@@ -1,5 +1,5 @@
 # ext/declarative/base.py
-# Copyright (C) 2005-2022 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2023 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
@@ -152,7 +152,13 @@ def _check_declared_props_nocascade(obj, name, cls):
 
 
 class _MapperConfig(object):
-    __slots__ = ("cls", "classname", "properties", "declared_attr_reg")
+    __slots__ = (
+        "cls",
+        "classname",
+        "properties",
+        "declared_attr_reg",
+        "__weakref__",
+    )
 
     @classmethod
     def setup_mapping(cls, registry, cls_, dict_, table, mapper_kw):
@@ -300,9 +306,12 @@ class _ClassScanMapperConfig(_MapperConfig):
         mapper_kw,
     ):
 
+        # grab class dict before the instrumentation manager has been added.
+        # reduces cycles
+        self.dict_ = dict(dict_) if dict_ else {}
+
         super(_ClassScanMapperConfig, self).__init__(registry, cls_, mapper_kw)
 
-        self.dict_ = dict(dict_) if dict_ else {}
         self.persist_selectable = None
         self.declared_columns = set()
         self.column_copies = {}
@@ -450,7 +459,14 @@ class _ClassScanMapperConfig(_MapperConfig):
 
         attribute_is_overridden = self._cls_attr_override_checker(self.cls)
 
+        bases = []
+
         for base in cls.__mro__:
+            # collect bases and make sure standalone columns are copied
+            # to be the column they will ultimately be on the class,
+            # so that declared_attr functions use the right columns.
+            # need to do this all the way up the hierarchy first
+            # (see #8190)
 
             class_mapped = (
                 base is not cls
@@ -463,9 +479,34 @@ class _ClassScanMapperConfig(_MapperConfig):
             local_attributes_for_class = self._cls_attr_resolver(base)
 
             if not class_mapped and base is not cls:
-                self._produce_column_copies(
-                    local_attributes_for_class, attribute_is_overridden
+                locally_collected_columns = self._produce_column_copies(
+                    local_attributes_for_class,
+                    attribute_is_overridden,
                 )
+            else:
+                locally_collected_columns = {}
+
+            bases.append(
+                (
+                    base,
+                    class_mapped,
+                    local_attributes_for_class,
+                    locally_collected_columns,
+                )
+            )
+
+        for (
+            base,
+            class_mapped,
+            local_attributes_for_class,
+            locally_collected_columns,
+        ) in bases:
+
+            # this transfer can also take place as we scan each name
+            # for finer-grained control of how collected_attributes is
+            # populated, as this is what impacts column ordering.
+            # however it's simpler to get it out of the way here.
+            dict_.update(locally_collected_columns)
 
             for name, obj, is_dataclass in local_attributes_for_class():
                 if name == "__mapper_args__":
@@ -631,6 +672,7 @@ class _ClassScanMapperConfig(_MapperConfig):
     ):
         cls = self.cls
         dict_ = self.dict_
+        locally_collected_attributes = {}
         column_copies = self.column_copies
         # copy mixin columns to the mapped class
 
@@ -655,7 +697,8 @@ class _ClassScanMapperConfig(_MapperConfig):
                     column_copies[obj] = copy_ = obj._copy()
                     copy_._creation_order = obj._creation_order
                     setattr(cls, name, copy_)
-                    dict_[name] = copy_
+                    locally_collected_attributes[name] = copy_
+        return locally_collected_attributes
 
     def _extract_mappable_attributes(self):
         cls = self.cls
