@@ -1,9 +1,9 @@
 # orm/state.py
-# Copyright (C) 2005-2019 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2023 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
-# the MIT License: http://www.opensource.org/licenses/mit-license.php
+# the MIT License: https://www.opensource.org/licenses/mit-license.php
 
 """Defines instrumentation of instances.
 
@@ -31,8 +31,15 @@ from .. import inspection
 from .. import util
 
 
+# late-populated by session.py
+_sessions = None
+
+# optionally late-provided by sqlalchemy.ext.asyncio.session
+_async_provider = None
+
+
 @inspection._self_inspects
-class InstanceState(interfaces.InspectionAttr):
+class InstanceState(interfaces.InspectionAttrInfo):
     """tracks state information at the instance level.
 
     The :class:`.InstanceState` is a key object used by the
@@ -47,22 +54,24 @@ class InstanceState(interfaces.InspectionAttr):
     status within a particular :class:`.Session` and details
     about data on individual attributes.  The public API
     in order to acquire a :class:`.InstanceState` object
-    is to use the :func:`.inspect` system::
+    is to use the :func:`_sa.inspect` system::
 
         >>> from sqlalchemy import inspect
         >>> insp = inspect(some_mapped_object)
+        >>> insp.attrs.nickname.history
+        History(added=['new nickname'], unchanged=(), deleted=['nickname'])
 
     .. seealso::
 
-        :ref:`core_inspection_toplevel`
+        :ref:`orm_mapper_inspection_instancestate`
 
     """
 
     session_id = None
     key = None
     runid = None
-    load_options = util.EMPTY_SET
-    load_path = ()
+    load_options = ()
+    load_path = PathRegistry.root
     insert_order = None
     _strong_obj = None
     modified = False
@@ -72,6 +81,7 @@ class InstanceState(interfaces.InspectionAttr):
     _orphaned_outside_of_session = False
     is_instance = True
     identity_token = None
+    _last_known_values = ()
 
     callables = ()
     """A namespace where a per-state loader callable can be associated.
@@ -118,7 +128,7 @@ class InstanceState(interfaces.InspectionAttr):
 
     @property
     def transient(self):
-        """Return true if the object is :term:`transient`.
+        """Return ``True`` if the object is :term:`transient`.
 
         .. seealso::
 
@@ -129,7 +139,7 @@ class InstanceState(interfaces.InspectionAttr):
 
     @property
     def pending(self):
-        """Return true if the object is :term:`pending`.
+        """Return ``True`` if the object is :term:`pending`.
 
 
         .. seealso::
@@ -141,7 +151,7 @@ class InstanceState(interfaces.InspectionAttr):
 
     @property
     def deleted(self):
-        """Return true if the object is :term:`deleted`.
+        """Return ``True`` if the object is :term:`deleted`.
 
         An object that is in the deleted state is guaranteed to
         not be within the :attr:`.Session.identity_map` of its parent
@@ -195,7 +205,7 @@ class InstanceState(interfaces.InspectionAttr):
 
     @property
     def persistent(self):
-        """Return true if the object is :term:`persistent`.
+        """Return ``True`` if the object is :term:`persistent`.
 
         An object that is in the persistent state is guaranteed to
         be within the :attr:`.Session.identity_map` of its parent
@@ -211,12 +221,12 @@ class InstanceState(interfaces.InspectionAttr):
 
             :ref:`session_object_states`
 
-            """
+        """
         return self.key is not None and self._attached and not self._deleted
 
     @property
     def detached(self):
-        """Return true if the object is :term:`detached`.
+        """Return ``True`` if the object is :term:`detached`.
 
         .. seealso::
 
@@ -226,16 +236,27 @@ class InstanceState(interfaces.InspectionAttr):
         return self.key is not None and not self._attached
 
     @property
-    @util.dependencies("sqlalchemy.orm.session")
-    def _attached(self, sessionlib):
+    @util.preload_module("sqlalchemy.orm.session")
+    def _attached(self):
         return (
             self.session_id is not None
-            and self.session_id in sessionlib._sessions
+            and self.session_id in util.preloaded.orm_session._sessions
         )
 
+    def _track_last_known_value(self, key):
+        """Track the last known value of a particular key after expiration
+        operations.
+
+        .. versionadded:: 1.3
+
+        """
+
+        if key not in self._last_known_values:
+            self._last_known_values = dict(self._last_known_values)
+            self._last_known_values[key] = NO_VALUE
+
     @property
-    @util.dependencies("sqlalchemy.orm.session")
-    def session(self, sessionlib):
+    def session(self):
         """Return the owning :class:`.Session` for this instance,
         or ``None`` if none available.
 
@@ -246,8 +267,45 @@ class InstanceState(interfaces.InspectionAttr):
         Only when the transaction is completed does the object become
         fully detached under normal circumstances.
 
+        .. seealso::
+
+            :attr:`_orm.InstanceState.async_session`
+
         """
-        return sessionlib._state_session(self)
+        if self.session_id:
+            try:
+                return _sessions[self.session_id]
+            except KeyError:
+                pass
+        return None
+
+    @property
+    def async_session(self):
+        """Return the owning :class:`_asyncio.AsyncSession` for this instance,
+        or ``None`` if none available.
+
+        This attribute is only non-None when the :mod:`sqlalchemy.ext.asyncio`
+        API is in use for this ORM object. The returned
+        :class:`_asyncio.AsyncSession` object will be a proxy for the
+        :class:`_orm.Session` object that would be returned from the
+        :attr:`_orm.InstanceState.session` attribute for this
+        :class:`_orm.InstanceState`.
+
+        .. versionadded:: 1.4.18
+
+        .. seealso::
+
+            :ref:`asyncio_toplevel`
+
+        """
+        if _async_provider is None:
+            return None
+
+        sess = self.session
+        if sess is not None:
+            return _async_provider(sess)
+        else:
+            return None
 
     @property
     def object(self):
@@ -260,7 +318,7 @@ class InstanceState(interfaces.InspectionAttr):
         """Return the mapped identity of the mapped object.
         This is the primary key identity as persisted by the ORM
         which can always be passed directly to
-        :meth:`.Query.get`.
+        :meth:`_query.Query.get`.
 
         Returns ``None`` if the object has no primary key identity.
 
@@ -297,9 +355,13 @@ class InstanceState(interfaces.InspectionAttr):
     def _pending_mutations(self):
         return {}
 
+    @util.memoized_property
+    def _empty_collections(self):
+        return {}
+
     @util.memoized_property
     def mapper(self):
-        """Return the :class:`.Mapper` used for this mapped object."""
+        """Return the :class:`_orm.Mapper` used for this mapped object."""
         return self.manager.mapper
 
     @property
@@ -307,7 +369,7 @@ class InstanceState(interfaces.InspectionAttr):
         """Return ``True`` if this object has an identity key.
 
         This should always have the same value as the
-        expression ``state.persistent or state.detached``.
+        expression ``state.persistent`` or ``state.detached``.
 
         """
         return bool(self.key)
@@ -335,21 +397,13 @@ class InstanceState(interfaces.InspectionAttr):
             if persistent:
                 if to_transient:
                     if persistent_to_transient is not None:
-                        obj = state.obj()
-                        if obj is not None:
-                            persistent_to_transient(session, obj)
+                        persistent_to_transient(session, state)
                 elif persistent_to_detached is not None:
-                    obj = state.obj()
-                    if obj is not None:
-                        persistent_to_detached(session, obj)
+                    persistent_to_detached(session, state)
             elif deleted and deleted_to_detached is not None:
-                obj = state.obj()
-                if obj is not None:
-                    deleted_to_detached(session, obj)
+                deleted_to_detached(session, state)
             elif pending and pending_to_transient is not None:
-                obj = state.obj()
-                if obj is not None:
-                    pending_to_transient(session, obj)
+                pending_to_transient(session, state)
 
             state._strong_obj = None
 
@@ -453,6 +507,7 @@ class InstanceState(interfaces.InspectionAttr):
                 "load_options",
                 "class_",
                 "expired_attributes",
+                "info",
             )
             if k in self.__dict__
         )
@@ -480,6 +535,8 @@ class InstanceState(interfaces.InspectionAttr):
         self.parents = state_dict.get("parents", {})
         self.modified = state_dict.get("modified", False)
         self.expired = state_dict.get("expired", False)
+        if "info" in state_dict:
+            self.info.update(state_dict["info"])
         if "callables" in state_dict:
             self.callables = state_dict["callables"]
 
@@ -521,7 +578,7 @@ class InstanceState(interfaces.InspectionAttr):
 
     def _reset(self, dict_, key):
         """Remove the given attribute and any
-           callables associated with it."""
+        callables associated with it."""
 
         old = dict_.pop(key, None)
         if old is not None and self.manager[key].impl.collection:
@@ -558,7 +615,6 @@ class InstanceState(interfaces.InspectionAttr):
 
     def _expire(self, dict_, modified_set):
         self.expired = True
-
         if self.modified:
             modified_set.discard(self)
             self.committed_state.clear()
@@ -573,14 +629,21 @@ class InstanceState(interfaces.InspectionAttr):
             del self.__dict__["parents"]
 
         self.expired_attributes.update(
-            [
-                impl.key
-                for impl in self.manager._scalar_loader_impls
-                if impl.expire_missing or impl.key in dict_
-            ]
+            [impl.key for impl in self.manager._loader_impls]
         )
 
         if self.callables:
+            # the per state loader callables we can remove here are
+            # LoadDeferredColumns, which undefers a column at the instance
+            # level that is mapped with deferred, and LoadLazyAttribute,
+            # which lazy loads a relationship at the instance level that
+            # is mapped with "noload" or perhaps "immediateload".
+            # Before 1.4, only column-based
+            # attributes could be considered to be "expired", so here they
+            # were the only ones "unexpired", which means to make them deferred
+            # again.   For the moment, as of 1.4 we also apply the same
+            # treatment relationships now, that is, an instance level lazy
+            # loader is reset in the same way as a column loader.
             for k in self.expired_attributes.intersection(self.callables):
                 del self.callables[k]
 
@@ -588,6 +651,11 @@ class InstanceState(interfaces.InspectionAttr):
             collection = dict_.pop(k)
             collection._sa_adapter.invalidated = True
 
+        if self._last_known_values:
+            self._last_known_values.update(
+                (k, dict_[k]) for k in self._last_known_values if k in dict_
+            )
+
         for key in self.manager._all_key_set.intersection(dict_):
             del dict_[key]
 
@@ -607,10 +675,17 @@ class InstanceState(interfaces.InspectionAttr):
                 self.expired_attributes.add(key)
                 if callables and key in callables:
                     del callables[key]
-            old = dict_.pop(key, None)
-            if impl.collection and old is not None:
+            old = dict_.pop(key, NO_VALUE)
+            if impl.collection and old is not NO_VALUE:
                 impl._invalidate_collection(old)
 
+            if (
+                self._last_known_values
+                and key in self._last_known_values
+                and old is not NO_VALUE
+            ):
+                self._last_known_values[key] = old
+
             self.committed_state.pop(key, None)
             if pending:
                 pending.pop(key, None)
@@ -628,8 +703,13 @@ class InstanceState(interfaces.InspectionAttr):
             return PASSIVE_NO_RESULT
 
         toload = self.expired_attributes.intersection(self.unmodified)
+        toload = toload.difference(
+            attr
+            for attr in toload
+            if not self.manager[attr].impl.load_on_unexpire
+        )
 
-        self.manager.deferred_scalar_loader(self, toload)
+        self.manager.expired_attribute_loader(self, toload, passive)
 
         # if the loader failed, or this
         # instance state didn't have an identity,
@@ -676,11 +756,7 @@ class InstanceState(interfaces.InspectionAttr):
         was never populated or modified.
 
         """
-        return self.unloaded.intersection(
-            attr
-            for attr in self.manager
-            if self.manager[attr].impl.expire_missing
-        )
+        return self.unloaded
 
     @property
     def _unloaded_non_object(self):
@@ -714,13 +790,19 @@ class InstanceState(interfaces.InspectionAttr):
                         previous = attr.copy(previous)
                 self.committed_state[attr.key] = previous
 
+            if attr.key in self._last_known_values:
+                self._last_known_values[attr.key] = NO_VALUE
+
         # assert self._strong_obj is None or self.modified
 
         if (self.session_id and self._strong_obj is None) or not self.modified:
             self.modified = True
             instance_dict = self._instance_dict()
             if instance_dict:
+                has_modified = bool(instance_dict._modified)
                 instance_dict._modified.add(self)
+            else:
+                has_modified = False
 
             # only create _strong_obj link if attached
             # to a session
@@ -729,6 +811,20 @@ class InstanceState(interfaces.InspectionAttr):
             if self.session_id:
                 self._strong_obj = inst
 
+                # if identity map already had modified objects,
+                # assume autobegin already occurred, else check
+                # for autobegin
+                if not has_modified:
+                    # inline of autobegin, to ensure session transaction
+                    # snapshot is established
+                    try:
+                        session = _sessions[self.session_id]
+                    except KeyError:
+                        pass
+                    else:
+                        if session._transaction is None:
+                            session._autobegin()
+
             if inst is None and attr:
                 raise orm_exc.ObjectDereferencedError(
                     "Can't emit change event for attribute '%s' - "
@@ -860,7 +956,8 @@ class AttributeState(object):
             The attribute history system tracks changes on a **per flush
             basis**. Each time the :class:`.Session` is flushed, the history
             of each attribute is reset to empty.   The :class:`.Session` by
-            default autoflushes each time a :class:`.Query` is invoked.  For
+            default autoflushes each time a :class:`_query.Query` is invoked.
+            For
             options on how to control this, see :ref:`session_flushing`.
 
 
@@ -886,7 +983,8 @@ class AttributeState(object):
             The attribute history system tracks changes on a **per flush
             basis**. Each time the :class:`.Session` is flushed, the history
             of each attribute is reset to empty.   The :class:`.Session` by
-            default autoflushes each time a :class:`.Query` is invoked.  For
+            default autoflushes each time a :class:`_query.Query` is invoked.
+            For
             options on how to control this, see :ref:`session_flushing`.
 
         .. seealso::
