@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2022 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2023 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
@@ -115,6 +115,9 @@ class Load(Generative, LoaderOption):
         load._extra_criteria = ()
         return load
 
+    def _adapt_cached_option_to_uncached_option(self, context, uncached_opt):
+        return self._adjust_for_extra_criteria(context)
+
     def _generate_extra_criteria(self, context):
         """Apply the current bound parameters in a QueryContext to the
         immediate "extra_criteria" stored with this Load object.
@@ -515,7 +518,19 @@ class Load(Generative, LoaderOption):
                 "for 'unbound' loader options"
             )
         for opt in opts:
-            opt._apply_to_parent(self, apply_cache, bound)
+            try:
+                opt._apply_to_parent(self, apply_cache, bound)
+            except AttributeError as ae:
+                if not isinstance(opt, Load):
+                    util.raise_(
+                        sa_exc.ArgumentError(
+                            "Loader option %s is not compatible with the "
+                            "Load.options() method." % (opt,)
+                        ),
+                        from_=ae,
+                    )
+                else:
+                    raise
 
     @_generative
     def set_relationship_strategy(
