@@ -33,7 +33,23 @@ from ...testing import is_true
 metadata, users = None, None
 
 
-class HasTableTest(fixtures.TablesTest):
+class OneConnectionTablesTest(fixtures.TablesTest):
+    @classmethod
+    def setup_bind(cls):
+        # TODO: when temp tables are subject to server reset,
+        # this will also have to disable that server reset from
+        # happening
+        if config.requirements.independent_connections.enabled:
+            from sqlalchemy import pool
+
+            return engines.testing_engine(
+                options=dict(poolclass=pool.StaticPool, scope="class"),
+            )
+        else:
+            return config.db
+
+
+class HasTableTest(OneConnectionTablesTest):
     __backend__ = True
 
     @classmethod
@@ -53,6 +69,64 @@ class HasTableTest(fixtures.TablesTest):
                 schema=config.test_schema,
             )
 
+        if testing.requires.view_reflection:
+            cls.define_views(metadata)
+        if testing.requires.has_temp_table.enabled:
+            cls.define_temp_tables(metadata)
+
+    @classmethod
+    def define_views(cls, metadata):
+        query = "CREATE VIEW vv AS SELECT id, data FROM test_table"
+
+        event.listen(metadata, "after_create", DDL(query))
+        event.listen(metadata, "before_drop", DDL("DROP VIEW vv"))
+
+        if testing.requires.schemas.enabled:
+            query = (
+                "CREATE VIEW %s.vv AS SELECT id, data FROM %s.test_table_s"
+                % (
+                    config.test_schema,
+                    config.test_schema,
+                )
+            )
+            event.listen(metadata, "after_create", DDL(query))
+            event.listen(
+                metadata,
+                "before_drop",
+                DDL("DROP VIEW %s.vv" % (config.test_schema)),
+            )
+
+    @classmethod
+    def temp_table_name(cls):
+        return get_temp_table_name(
+            config, config.db, "user_tmp_%s" % (config.ident,)
+        )
+
+    @classmethod
+    def define_temp_tables(cls, metadata):
+        kw = temp_table_keyword_args(config, config.db)
+        table_name = cls.temp_table_name()
+        user_tmp = Table(
+            table_name,
+            metadata,
+            Column("id", sa.INT, primary_key=True),
+            Column("name", sa.VARCHAR(50)),
+            **kw
+        )
+        if (
+            testing.requires.view_reflection.enabled
+            and testing.requires.temporary_views.enabled
+        ):
+            event.listen(
+                user_tmp,
+                "after_create",
+                DDL(
+                    "create temporary view user_tmp_v as "
+                    "select * from user_tmp_%s" % config.ident
+                ),
+            )
+            event.listen(user_tmp, "before_drop", DDL("drop view user_tmp_v"))
+
     def test_has_table(self):
         with config.db.begin() as conn:
             is_true(config.db.dialect.has_table(conn, "test_table"))
@@ -78,6 +152,40 @@ class HasTableTest(fixtures.TablesTest):
                 )
             )
 
+    @testing.fails_on(
+        "oracle",
+        "per #8700 this remains at its previous behavior of not "
+        "working within 1.4.",
+    )
+    @testing.requires.views
+    def test_has_table_view(self, connection):
+        insp = inspect(connection)
+        is_true(insp.has_table("vv"))
+
+    @testing.requires.has_temp_table
+    def test_has_table_temp_table(self, connection):
+        insp = inspect(connection)
+        temp_table_name = self.temp_table_name()
+        is_true(insp.has_table(temp_table_name))
+
+    @testing.requires.has_temp_table
+    @testing.requires.view_reflection
+    @testing.requires.temporary_views
+    def test_has_table_temp_view(self, connection):
+        insp = inspect(connection)
+        is_true(insp.has_table("user_tmp_v"))
+
+    @testing.fails_on(
+        "oracle",
+        "per #8700 this remains at its previous behavior of not "
+        "working within 1.4",
+    )
+    @testing.requires.views
+    @testing.requires.schemas
+    def test_has_table_view_schema(self, connection):
+        insp = inspect(connection)
+        is_true(insp.has_table("vv", config.test_schema))
+
 
 class HasIndexTest(fixtures.TablesTest):
     __backend__ = True
@@ -283,22 +391,11 @@ class QuotedNameArgumentTest(fixtures.TablesTest):
         assert insp.get_check_constraints(name)
 
 
-class ComponentReflectionTest(fixtures.TablesTest):
+class ComponentReflectionTest(OneConnectionTablesTest):
     run_inserts = run_deletes = None
 
     __backend__ = True
 
-    @classmethod
-    def setup_bind(cls):
-        if config.requirements.independent_connections.enabled:
-            from sqlalchemy import pool
-
-            return engines.testing_engine(
-                options=dict(poolclass=pool.StaticPool, scope="class"),
-            )
-        else:
-            return config.db
-
     @classmethod
     def define_tables(cls, metadata):
         cls.define_reflected_tables(metadata, None)
@@ -985,6 +1082,8 @@ class ComponentReflectionTest(fixtures.TablesTest):
 
         names_that_duplicate_index = set()
 
+        eq_(len(uniques), len(reflected))
+
         for orig, refl in zip(uniques, reflected):
             # Different dialects handle duplicate index and constraints
             # differently, so ignore this flag
@@ -1152,7 +1251,9 @@ class ComponentReflectionTestExtra(fixtures.TestBase):
             metadata,
             Column("a", Integer()),
             sa.CheckConstraint("a > 1 AND a < 5", name="cc1"),
-            sa.CheckConstraint("a = 1 OR (a > 2 AND a < 5)", name="cc2"),
+            sa.CheckConstraint(
+                "a = 1 OR (a > 2 AND a < 5)", name="UsesCasing"
+            ),
             schema=schema,
         )
 
@@ -1179,8 +1280,8 @@ class ComponentReflectionTestExtra(fixtures.TestBase):
         eq_(
             reflected,
             [
+                {"name": "UsesCasing", "sqltext": "a = 1 or a > 2 and a < 5"},
                 {"name": "cc1", "sqltext": "a > 1 and a < 5"},
-                {"name": "cc2", "sqltext": "a = 1 or a > 2 and a < 5"},
             ],
         )
 
@@ -1202,8 +1303,14 @@ class ComponentReflectionTestExtra(fixtures.TestBase):
         insp = inspect(connection)
 
         expected = [
-            {"name": "t_idx_2", "column_names": ["x"], "unique": False}
+            {
+                "name": "t_idx_2",
+                "column_names": ["x"],
+                "unique": False,
+                "dialect_options": {},
+            }
         ]
+
         if testing.requires.index_reflects_included_columns.enabled:
             expected[0]["include_columns"] = []
             expected[0]["dialect_options"] = {
@@ -1213,10 +1320,7 @@ class ComponentReflectionTestExtra(fixtures.TestBase):
         with expect_warnings(
             "Skipped unsupported reflection of expression-based index t_idx"
         ):
-            eq_(
-                insp.get_indexes("t"),
-                expected,
-            )
+            eq_(insp.get_indexes("t"), expected)
 
     @testing.requires.index_reflects_included_columns
     def test_reflect_covering_index(self, metadata, connection):
