@@ -4,8 +4,12 @@ import pickle
 from sqlalchemy import event
 from sqlalchemy import ForeignKey
 from sqlalchemy import func
+from sqlalchemy import inspect
 from sqlalchemy import Integer
+from sqlalchemy import JSON
+from sqlalchemy import select
 from sqlalchemy import String
+from sqlalchemy import testing
 from sqlalchemy import util
 from sqlalchemy.ext.mutable import MutableComposite
 from sqlalchemy.ext.mutable import MutableDict
@@ -14,7 +18,7 @@ from sqlalchemy.ext.mutable import MutableSet
 from sqlalchemy.orm import attributes
 from sqlalchemy.orm import column_property
 from sqlalchemy.orm import composite
-from sqlalchemy.orm import mapper
+from sqlalchemy.orm import declarative_base
 from sqlalchemy.orm import Session
 from sqlalchemy.orm.instrumentation import ClassManager
 from sqlalchemy.orm.mapper import Mapper
@@ -22,7 +26,10 @@ from sqlalchemy.testing import assert_raises
 from sqlalchemy.testing import assert_raises_message
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import fixtures
+from sqlalchemy.testing import is_
+from sqlalchemy.testing import is_true
 from sqlalchemy.testing import mock
+from sqlalchemy.testing.fixtures import fixture_session
 from sqlalchemy.testing.schema import Column
 from sqlalchemy.testing.schema import Table
 from sqlalchemy.testing.util import picklers
@@ -39,6 +46,10 @@ class SubFoo(Foo):
     pass
 
 
+class Foo2(fixtures.BasicEntity):
+    pass
+
+
 class FooWithEq(object):
     def __init__(self, **kw):
         for k in kw:
@@ -51,6 +62,10 @@ class FooWithEq(object):
         return self.id == other.id
 
 
+class FooWNoHash(fixtures.BasicEntity):
+    __hash__ = None
+
+
 class Point(MutableComposite):
     def __init__(self, x, y):
         self.x = x
@@ -90,11 +105,105 @@ class _MutableDictTestFixture(object):
     def _type_fixture(cls):
         return MutableDict
 
-    def teardown(self):
+    def teardown_test(self):
         # clear out mapper events
         Mapper.dispatch._clear()
         ClassManager.dispatch._clear()
-        super(_MutableDictTestFixture, self).teardown()
+
+
+class MiscTest(fixtures.TestBase):
+    @testing.combinations(True, False, argnames="pickleit")
+    def test_pickle_parent_multi_attrs(self, registry, connection, pickleit):
+        """test #8133"""
+
+        local_foo = Table(
+            "lf",
+            registry.metadata,
+            Column("id", Integer, primary_key=True),
+            Column("j1", MutableDict.as_mutable(PickleType)),
+            Column("j2", MutableDict.as_mutable(PickleType)),
+            Column("j3", MutableDict.as_mutable(PickleType)),
+            Column("j4", MutableDict.as_mutable(PickleType)),
+        )
+
+        registry.map_imperatively(Foo2, local_foo)
+        registry.metadata.create_all(connection)
+
+        with Session(connection) as sess:
+
+            data = dict(
+                j1={"a": 1},
+                j2={"b": 2},
+                j3={"c": 3},
+                j4={"d": 4},
+            )
+            lf = Foo2(**data)
+            sess.add(lf)
+            sess.commit()
+
+        all_attrs = {"j1", "j2", "j3", "j4"}
+        for attr in all_attrs:
+            for loads, dumps in picklers():
+                with Session(connection) as sess:
+                    f1 = sess.scalars(select(Foo2)).first()
+                    if pickleit:
+                        f2 = loads(dumps(f1))
+                    else:
+                        f2 = f1
+
+                existing_dict = getattr(f2, attr)
+                existing_dict["q"] = "c"
+                eq_(
+                    inspect(f2).attrs[attr].history,
+                    ([existing_dict], (), ()),
+                )
+                for other_attr in all_attrs.difference([attr]):
+                    a = inspect(f2).attrs[other_attr].history
+                    b = ((), [data[other_attr]], ())
+                    eq_(a, b)
+
+    @testing.combinations("key_present", "key_non_present", argnames="present")
+    @testing.combinations(
+        ("transient", True),
+        ("detached", True),
+        ("detached", False),
+        argnames="merge_subject, load",
+    )
+    @testing.requires.json_type
+    def test_session_merge(
+        self, decl_base, connection, present, load, merge_subject
+    ):
+        """test #8446"""
+
+        class Thing(decl_base):
+            __tablename__ = "thing"
+            id = Column(Integer, primary_key=True)
+            data = Column(MutableDict.as_mutable(JSON))
+
+        decl_base.metadata.create_all(connection)
+
+        with Session(connection) as sess:
+            sess.add(Thing(id=1, data={"foo": "bar"}))
+            sess.commit()
+
+        if merge_subject == "transient":
+            t1_to_merge = Thing(id=1, data={"foo": "bar"})
+        elif merge_subject == "detached":
+            with Session(connection) as sess:
+                t1_to_merge = sess.get(Thing, 1)
+
+        with Session(connection) as sess:
+            already_present = None
+            if present == "key_present":
+                already_present = sess.get(Thing, 1)
+
+            t1_merged = sess.merge(t1_to_merge, load=load)
+
+            t1_merged.data["foo"] = "bat"
+            if present == "key_present":
+                is_(t1_merged, already_present)
+
+            is_true(inspect(t1_merged).attrs.data.history.added)
 
 
 class _MutableDictTestBase(_MutableDictTestFixture):
@@ -103,10 +212,10 @@ class _MutableDictTestBase(_MutableDictTestFixture):
     def setup_mappers(cls):
         foo = cls.tables.foo
 
-        mapper(Foo, foo)
+        cls.mapper_registry.map_imperatively(Foo, foo)
 
     def test_coerce_none(self):
-        sess = Session()
+        sess = fixture_session()
         f1 = Foo(data=None)
         sess.add(f1)
         sess.commit()
@@ -121,7 +230,7 @@ class _MutableDictTestBase(_MutableDictTestFixture):
         )
 
     def test_in_place_mutation(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data={"a": "b"})
         sess.add(f1)
@@ -149,7 +258,7 @@ class _MutableDictTestBase(_MutableDictTestFixture):
         )
 
     def test_clear(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data={"a": "b"})
         sess.add(f1)
@@ -161,7 +270,7 @@ class _MutableDictTestBase(_MutableDictTestFixture):
         eq_(f1.data, {})
 
     def test_update(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data={"a": "b"})
         sess.add(f1)
@@ -173,7 +282,7 @@ class _MutableDictTestBase(_MutableDictTestFixture):
         eq_(f1.data, {"a": "z"})
 
     def test_pop(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data={"a": "b", "c": "d"})
         sess.add(f1)
@@ -187,7 +296,7 @@ class _MutableDictTestBase(_MutableDictTestFixture):
         eq_(f1.data, {"c": "d"})
 
     def test_pop_default(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data={"a": "b", "c": "d"})
         sess.add(f1)
@@ -200,7 +309,7 @@ class _MutableDictTestBase(_MutableDictTestFixture):
         eq_(f1.data, {"c": "d"})
 
     def test_popitem(self):
-        sess = Session()
+        sess = fixture_session()
 
         orig = {"a": "b", "c": "d"}
 
@@ -220,7 +329,7 @@ class _MutableDictTestBase(_MutableDictTestFixture):
         eq_(f1.data, orig)
 
     def test_setdefault(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data={"a": "b"})
         sess.add(f1)
@@ -237,7 +346,7 @@ class _MutableDictTestBase(_MutableDictTestFixture):
         eq_(f1.data, {"a": "b", "c": "d"})
 
     def test_replace(self):
-        sess = Session()
+        sess = fixture_session()
         f1 = Foo(data={"a": "b"})
         sess.add(f1)
         sess.flush()
@@ -247,7 +356,7 @@ class _MutableDictTestBase(_MutableDictTestFixture):
         eq_(f1.data, {"b": "c"})
 
     def test_replace_itself_still_ok(self):
-        sess = Session()
+        sess = fixture_session()
         f1 = Foo(data={"a": "b"})
         sess.add(f1)
         sess.flush()
@@ -258,7 +367,7 @@ class _MutableDictTestBase(_MutableDictTestFixture):
         eq_(f1.data, {"a": "b", "b": "c"})
 
     def test_pickle_parent(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data={"a": "b"})
         sess.add(f1)
@@ -267,14 +376,14 @@ class _MutableDictTestBase(_MutableDictTestFixture):
         sess.close()
 
         for loads, dumps in picklers():
-            sess = Session()
+            sess = fixture_session()
             f2 = loads(dumps(f1))
             sess.add(f2)
             f2.data["a"] = "c"
             assert f2 in sess.dirty
 
     def test_unrelated_flush(self):
-        sess = Session()
+        sess = fixture_session()
         f1 = Foo(data={"a": "b"}, unrelated_data="unrelated")
         sess.add(f1)
         sess.flush()
@@ -285,7 +394,7 @@ class _MutableDictTestBase(_MutableDictTestFixture):
         eq_(f1.data["a"], "c")
 
     def _test_non_mutable(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(non_mutable_data={"a": "b"})
         sess.add(f1)
@@ -312,11 +421,10 @@ class _MutableListTestFixture(object):
     def _type_fixture(cls):
         return MutableList
 
-    def teardown(self):
+    def teardown_test(self):
         # clear out mapper events
         Mapper.dispatch._clear()
         ClassManager.dispatch._clear()
-        super(_MutableListTestFixture, self).teardown()
 
 
 class _MutableListTestBase(_MutableListTestFixture):
@@ -325,10 +433,10 @@ class _MutableListTestBase(_MutableListTestFixture):
     def setup_mappers(cls):
         foo = cls.tables.foo
 
-        mapper(Foo, foo)
+        cls.mapper_registry.map_imperatively(Foo, foo)
 
     def test_coerce_none(self):
-        sess = Session()
+        sess = fixture_session()
         f1 = Foo(data=None)
         sess.add(f1)
         sess.commit()
@@ -343,7 +451,7 @@ class _MutableListTestBase(_MutableListTestFixture):
         )
 
     def test_in_place_mutation(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=[1, 2])
         sess.add(f1)
@@ -355,7 +463,7 @@ class _MutableListTestBase(_MutableListTestFixture):
         eq_(f1.data, [3, 2])
 
     def test_in_place_slice_mutation(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=[1, 2, 3, 4])
         sess.add(f1)
@@ -367,7 +475,7 @@ class _MutableListTestBase(_MutableListTestFixture):
         eq_(f1.data, [1, 5, 6, 4])
 
     def test_del_slice(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=[1, 2, 3, 4])
         sess.add(f1)
@@ -382,7 +490,7 @@ class _MutableListTestBase(_MutableListTestFixture):
         if not hasattr(list, "clear"):
             # py2 list doesn't have 'clear'
             return
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=[1, 2])
         sess.add(f1)
@@ -394,7 +502,7 @@ class _MutableListTestBase(_MutableListTestFixture):
         eq_(f1.data, [])
 
     def test_pop(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=[1, 2, 3])
         sess.add(f1)
@@ -409,7 +517,7 @@ class _MutableListTestBase(_MutableListTestFixture):
         eq_(f1.data, [2])
 
     def test_append(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=[1, 2])
         sess.add(f1)
@@ -421,7 +529,7 @@ class _MutableListTestBase(_MutableListTestFixture):
         eq_(f1.data, [1, 2, 5])
 
     def test_extend(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=[1, 2])
         sess.add(f1)
@@ -433,7 +541,7 @@ class _MutableListTestBase(_MutableListTestFixture):
         eq_(f1.data, [1, 2, 5])
 
     def test_operator_extend(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=[1, 2])
         sess.add(f1)
@@ -445,7 +553,7 @@ class _MutableListTestBase(_MutableListTestFixture):
         eq_(f1.data, [1, 2, 5])
 
     def test_insert(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=[1, 2])
         sess.add(f1)
@@ -457,7 +565,7 @@ class _MutableListTestBase(_MutableListTestFixture):
         eq_(f1.data, [1, 5, 2])
 
     def test_remove(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=[1, 2, 3])
         sess.add(f1)
@@ -469,7 +577,7 @@ class _MutableListTestBase(_MutableListTestFixture):
         eq_(f1.data, [1, 3])
 
     def test_sort(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=[1, 3, 2])
         sess.add(f1)
@@ -481,7 +589,7 @@ class _MutableListTestBase(_MutableListTestFixture):
         eq_(f1.data, [1, 2, 3])
 
     def test_sort_w_key(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=[1, 3, 2])
         sess.add(f1)
@@ -493,7 +601,7 @@ class _MutableListTestBase(_MutableListTestFixture):
         eq_(f1.data, [3, 2, 1])
 
     def test_sort_w_reverse_kwarg(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=[1, 3, 2])
         sess.add(f1)
@@ -505,7 +613,7 @@ class _MutableListTestBase(_MutableListTestFixture):
         eq_(f1.data, [3, 2, 1])
 
     def test_reverse(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=[1, 3, 2])
         sess.add(f1)
@@ -517,7 +625,7 @@ class _MutableListTestBase(_MutableListTestFixture):
         eq_(f1.data, [2, 3, 1])
 
     def test_pickle_parent(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=[1, 2])
         sess.add(f1)
@@ -526,14 +634,14 @@ class _MutableListTestBase(_MutableListTestFixture):
         sess.close()
 
         for loads, dumps in picklers():
-            sess = Session()
+            sess = fixture_session()
             f2 = loads(dumps(f1))
             sess.add(f2)
             f2.data[0] = 3
             assert f2 in sess.dirty
 
     def test_unrelated_flush(self):
-        sess = Session()
+        sess = fixture_session()
         f1 = Foo(data=[1, 2], unrelated_data="unrelated")
         sess.add(f1)
         sess.flush()
@@ -619,11 +727,10 @@ class _MutableSetTestFixture(object):
     def _type_fixture(cls):
         return MutableSet
 
-    def teardown(self):
+    def teardown_test(self):
         # clear out mapper events
         Mapper.dispatch._clear()
         ClassManager.dispatch._clear()
-        super(_MutableSetTestFixture, self).teardown()
 
 
 class _MutableSetTestBase(_MutableSetTestFixture):
@@ -632,10 +739,10 @@ class _MutableSetTestBase(_MutableSetTestFixture):
     def setup_mappers(cls):
         foo = cls.tables.foo
 
-        mapper(Foo, foo)
+        cls.mapper_registry.map_imperatively(Foo, foo)
 
     def test_coerce_none(self):
-        sess = Session()
+        sess = fixture_session()
         f1 = Foo(data=None)
         sess.add(f1)
         sess.commit()
@@ -650,7 +757,7 @@ class _MutableSetTestBase(_MutableSetTestFixture):
         )
 
     def test_clear(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=set([1, 2]))
         sess.add(f1)
@@ -662,7 +769,7 @@ class _MutableSetTestBase(_MutableSetTestFixture):
         eq_(f1.data, set())
 
     def test_pop(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=set([1]))
         sess.add(f1)
@@ -676,7 +783,7 @@ class _MutableSetTestBase(_MutableSetTestFixture):
         eq_(f1.data, set())
 
     def test_add(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=set([1, 2]))
         sess.add(f1)
@@ -688,7 +795,7 @@ class _MutableSetTestBase(_MutableSetTestFixture):
         eq_(f1.data, set([1, 2, 5]))
 
     def test_update(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=set([1, 2]))
         sess.add(f1)
@@ -700,7 +807,7 @@ class _MutableSetTestBase(_MutableSetTestFixture):
         eq_(f1.data, set([1, 2, 5]))
 
     def test_binary_update(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=set([1, 2]))
         sess.add(f1)
@@ -712,7 +819,7 @@ class _MutableSetTestBase(_MutableSetTestFixture):
         eq_(f1.data, set([1, 2, 5]))
 
     def test_intersection_update(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=set([1, 2]))
         sess.add(f1)
@@ -724,7 +831,7 @@ class _MutableSetTestBase(_MutableSetTestFixture):
         eq_(f1.data, set([2]))
 
     def test_binary_intersection_update(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=set([1, 2]))
         sess.add(f1)
@@ -736,7 +843,7 @@ class _MutableSetTestBase(_MutableSetTestFixture):
         eq_(f1.data, set([2]))
 
     def test_difference_update(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=set([1, 2]))
         sess.add(f1)
@@ -748,7 +855,7 @@ class _MutableSetTestBase(_MutableSetTestFixture):
         eq_(f1.data, set([1]))
 
     def test_operator_difference_update(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=set([1, 2]))
         sess.add(f1)
@@ -760,7 +867,7 @@ class _MutableSetTestBase(_MutableSetTestFixture):
         eq_(f1.data, set([1]))
 
     def test_symmetric_difference_update(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=set([1, 2]))
         sess.add(f1)
@@ -772,7 +879,7 @@ class _MutableSetTestBase(_MutableSetTestFixture):
         eq_(f1.data, set([1, 5]))
 
     def test_binary_symmetric_difference_update(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=set([1, 2]))
         sess.add(f1)
@@ -784,7 +891,7 @@ class _MutableSetTestBase(_MutableSetTestFixture):
         eq_(f1.data, set([1, 5]))
 
     def test_remove(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=set([1, 2, 3]))
         sess.add(f1)
@@ -796,7 +903,7 @@ class _MutableSetTestBase(_MutableSetTestFixture):
         eq_(f1.data, set([1, 3]))
 
     def test_discard(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=set([1, 2, 3]))
         sess.add(f1)
@@ -813,7 +920,7 @@ class _MutableSetTestBase(_MutableSetTestFixture):
         eq_(f1.data, set([1, 3]))
 
     def test_pickle_parent(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=set([1, 2]))
         sess.add(f1)
@@ -822,14 +929,14 @@ class _MutableSetTestBase(_MutableSetTestFixture):
         sess.close()
 
         for loads, dumps in picklers():
-            sess = Session()
+            sess = fixture_session()
             f2 = loads(dumps(f1))
             sess.add(f2)
             f2.data.add(3)
             assert f2 in sess.dirty
 
     def test_unrelated_flush(self):
-        sess = Session()
+        sess = fixture_session()
         f1 = Foo(data=set([1, 2]), unrelated_data="unrelated")
         sess.add(f1)
         sess.flush()
@@ -850,6 +957,60 @@ class _MutableSetTestBase(_MutableSetTestFixture):
         eq_(f1.data, set([1, 2]))
 
 
+class _MutableNoHashFixture(object):
+    @testing.fixture(autouse=True, scope="class")
+    def set_class(self):
+        global Foo
+
+        _replace_foo = Foo
+        Foo = FooWNoHash
+
+        yield
+        Foo = _replace_foo
+
+    def test_ensure_not_hashable(self):
+        d = {}
+        obj = Foo()
+        with testing.expect_raises(TypeError):
+            d[obj] = True
+
+
+class MutableListNoHashTest(
+    _MutableNoHashFixture, _MutableListTestBase, fixtures.MappedTest
+):
+    @classmethod
+    def define_tables(cls, metadata):
+        MutableList = cls._type_fixture()
+
+        mutable_pickle = MutableList.as_mutable(PickleType)
+        Table(
+            "foo",
+            metadata,
+            Column(
+                "id", Integer, primary_key=True, test_needs_autoincrement=True
+            ),
+            Column("data", mutable_pickle),
+        )
+
+
+class MutableDictNoHashTest(
+    _MutableNoHashFixture, _MutableDictTestBase, fixtures.MappedTest
+):
+    @classmethod
+    def define_tables(cls, metadata):
+        MutableDict = cls._type_fixture()
+
+        mutable_pickle = MutableDict.as_mutable(PickleType)
+        Table(
+            "foo",
+            metadata,
+            Column(
+                "id", Integer, primary_key=True, test_needs_autoincrement=True
+            ),
+            Column("data", mutable_pickle),
+        )
+
+
 class MutableColumnDefaultTest(_MutableDictTestFixture, fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
@@ -868,12 +1029,12 @@ class MutableColumnDefaultTest(_MutableDictTestFixture, fixtures.MappedTest):
     def setup_mappers(cls):
         foo = cls.tables.foo
 
-        mapper(Foo, foo)
+        cls.mapper_registry.map_imperatively(Foo, foo)
 
     def test_evt_on_flush_refresh(self):
         # test for #3427
 
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo()
         sess.add(f1)
@@ -913,6 +1074,7 @@ class MutableWithScalarJSONTest(_MutableDictTestBase, fixtures.MappedTest):
 
         class JSONEncodedDict(TypeDecorator):
             impl = VARCHAR(50)
+            cache_ok = True
 
             def process_bind_param(self, value, dialect):
                 if value is not None:
@@ -946,10 +1108,10 @@ class MutableColumnCopyJSONTest(_MutableDictTestBase, fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         import json
-        from sqlalchemy.ext.declarative import declarative_base
 
         class JSONEncodedDict(TypeDecorator):
             impl = VARCHAR(50)
+            cache_ok = True
 
             def process_bind_param(self, value, dialect):
                 if value is not None:
@@ -993,7 +1155,6 @@ class MutableColumnCopyArrayTest(_MutableListTestBase, fixtures.MappedTest):
 
     @classmethod
     def define_tables(cls, metadata):
-        from sqlalchemy.ext.declarative import declarative_base
         from sqlalchemy.sql.sqltypes import ARRAY
 
         MutableList = cls._type_fixture()
@@ -1075,12 +1236,12 @@ class MutableAssocWithAttrInheritTest(
         foo = cls.tables.foo
         subfoo = cls.tables.subfoo
 
-        mapper(Foo, foo)
-        mapper(SubFoo, subfoo, inherits=Foo)
+        cls.mapper_registry.map_imperatively(Foo, foo)
+        cls.mapper_registry.map_imperatively(SubFoo, subfoo, inherits=Foo)
         MutableDict.associate_with_attribute(Foo.data)
 
     def test_in_place_mutation(self):
-        sess = Session()
+        sess = fixture_session()
 
         f1 = SubFoo(data={"a": "b"})
         sess.add(f1)
@@ -1092,7 +1253,7 @@ class MutableAssocWithAttrInheritTest(
         eq_(f1.data, {"a": "c"})
 
     def test_replace(self):
-        sess = Session()
+        sess = fixture_session()
         f1 = SubFoo(data={"a": "b"})
         sess.add(f1)
         sess.flush()
@@ -1131,6 +1292,7 @@ class MutableAssociationScalarJSONTest(
 
         class JSONEncodedDict(TypeDecorator):
             impl = VARCHAR(50)
+            cache_ok = True
 
             def process_bind_param(self, value, dialect):
                 if value is not None:
@@ -1182,6 +1344,7 @@ class CustomMutableAssociationScalarJSONTest(
 
         class JSONEncodedDict(TypeDecorator):
             impl = VARCHAR(50)
+            cache_ok = True
 
             def process_bind_param(self, value, dialect):
                 if value is not None:
@@ -1213,7 +1376,7 @@ class CustomMutableAssociationScalarJSONTest(
         pass
 
     def test_coerce(self):
-        sess = Session()
+        sess = fixture_session()
         f1 = Foo(data={"a": "b"})
         sess.add(f1)
         sess.flush()
@@ -1234,17 +1397,15 @@ class _CompositeTestBase(object):
             Column("unrelated_data", String(50)),
         )
 
-    def setup(self):
+    def setup_test(self):
         from sqlalchemy.ext import mutable
 
         mutable._setup_composite_listener()
-        super(_CompositeTestBase, self).setup()
 
-    def teardown(self):
+    def teardown_test(self):
         # clear out mapper events
         Mapper.dispatch._clear()
         ClassManager.dispatch._clear()
-        super(_CompositeTestBase, self).teardown()
 
     @classmethod
     def _type_fixture(cls):
@@ -1274,7 +1435,7 @@ class MutableCompositeColumnDefaultTest(
 
         cls.Point = cls._type_fixture()
 
-        mapper(
+        cls.mapper_registry.map_imperatively(
             Foo,
             foo,
             properties={"data": composite(cls.Point, foo.c.x, foo.c.y)},
@@ -1283,7 +1444,7 @@ class MutableCompositeColumnDefaultTest(
     def test_evt_on_flush_refresh(self):
         # this still worked prior to #3427 being fixed in any case
 
-        sess = Session()
+        sess = fixture_session()
 
         f1 = Foo(data=self.Point(None, None))
         sess.add(f1)
@@ -1301,7 +1462,7 @@ class MutableCompositesUnpickleTest(_CompositeTestBase, fixtures.MappedTest):
 
         cls.Point = cls._type_fixture()
 
-        mapper(
+        cls.mapper_registry.map_imperatively(
             FooWithEq,
             foo,
             properties={"data": composite(cls.Point, foo.c.x, foo.c.y)},
@@ -1320,12 +1481,12 @@ class MutableCompositesTest(_CompositeTestBase, fixtures.MappedTest):
 
         Point = cls._type_fixture()
 
-        mapper(
+        cls.mapper_registry.map_imperatively(
             Foo, foo, properties={"data": composite(Point, foo.c.x, foo.c.y)}
         )
 
     def test_in_place_mutation(self):
-        sess = Session()
+        sess = fixture_session()
         d = Point(3, 4)
         f1 = Foo(data=d)
         sess.add(f1)
@@ -1337,7 +1498,7 @@ class MutableCompositesTest(_CompositeTestBase, fixtures.MappedTest):
         eq_(f1.data, Point(3, 5))
 
     def test_pickle_of_parent(self):
-        sess = Session()
+        sess = fixture_session()
         d = Point(3, 4)
         f1 = Foo(data=d)
         sess.add(f1)
@@ -1348,14 +1509,14 @@ class MutableCompositesTest(_CompositeTestBase, fixtures.MappedTest):
         sess.close()
 
         for loads, dumps in picklers():
-            sess = Session()
+            sess = fixture_session()
             f2 = loads(dumps(f1))
             sess.add(f2)
             f2.data.y = 12
             assert f2 in sess.dirty
 
     def test_set_none(self):
-        sess = Session()
+        sess = fixture_session()
         f1 = Foo(data=None)
         sess.add(f1)
         sess.commit()
@@ -1377,7 +1538,7 @@ class MutableCompositesTest(_CompositeTestBase, fixtures.MappedTest):
         )
 
     def test_unrelated_flush(self):
-        sess = Session()
+        sess = fixture_session()
         f1 = Foo(data=Point(3, 4), unrelated_data="unrelated")
         sess.add(f1)
         sess.flush()
@@ -1388,6 +1549,39 @@ class MutableCompositesTest(_CompositeTestBase, fixtures.MappedTest):
 
         eq_(f1.data.x, 5)
 
+    def test_dont_reset_on_attr_refresh(self):
+        sess = fixture_session()
+        f1 = Foo(data=Point(3, 4), unrelated_data="unrelated")
+        sess.add(f1)
+        sess.flush()
+
+        f1.data.x = 5
+
+        # issue 6001, this would reload a new Point() that would be missed
+        # by the mutable composite, and tracking would be lost
+        sess.refresh(f1, ["unrelated_data"])
+
+        is_(list(f1.data._parents.keys())[0], f1._sa_instance_state)
+
+        f1.data.y = 9
+
+        sess.commit()
+
+        eq_(f1.data.x, 5)
+        eq_(f1.data.y, 9)
+
+        f1.data.x = 12
+
+        sess.refresh(f1, ["unrelated_data", "y"])
+
+        is_(list(f1.data._parents.keys())[0], f1._sa_instance_state)
+
+        f1.data.y = 15
+        sess.commit()
+
+        eq_(f1.data.x, 12)
+        eq_(f1.data.y, 15)
+
 
 class MutableCompositeCallableTest(_CompositeTestBase, fixtures.MappedTest):
     @classmethod
@@ -1398,7 +1592,7 @@ class MutableCompositeCallableTest(_CompositeTestBase, fixtures.MappedTest):
 
         # in this case, this is not actually a MutableComposite.
         # so we don't expect it to track changes
-        mapper(
+        cls.mapper_registry.map_imperatively(
             Foo,
             foo,
             properties={
@@ -1407,7 +1601,7 @@ class MutableCompositeCallableTest(_CompositeTestBase, fixtures.MappedTest):
         )
 
     def test_basic(self):
-        sess = Session()
+        sess = fixture_session()
         f1 = Foo(data=Point(3, 4))
         sess.add(f1)
         sess.flush()
@@ -1432,7 +1626,7 @@ class MutableCompositeCustomCoerceTest(
 
         Point = cls._type_fixture()
 
-        mapper(
+        cls.mapper_registry.map_imperatively(
             Foo, foo, properties={"data": composite(Point, foo.c.x, foo.c.y)}
         )
 
@@ -1442,7 +1636,7 @@ class MutableCompositeCustomCoerceTest(
         eq_(f.data, Point(3, 4))
 
     def test_round_trip_ok(self):
-        sess = Session()
+        sess = fixture_session()
         f = Foo()
         f.data = (3, 4)
 
@@ -1477,13 +1671,13 @@ class MutableInheritedCompositesTest(_CompositeTestBase, fixtures.MappedTest):
 
         Point = cls._type_fixture()
 
-        mapper(
+        cls.mapper_registry.map_imperatively(
             Foo, foo, properties={"data": composite(Point, foo.c.x, foo.c.y)}
         )
-        mapper(SubFoo, subfoo, inherits=Foo)
+        cls.mapper_registry.map_imperatively(SubFoo, subfoo, inherits=Foo)
 
     def test_in_place_mutation_subclass(self):
-        sess = Session()
+        sess = fixture_session()
         d = Point(3, 4)
         f1 = SubFoo(data=d)
         sess.add(f1)
@@ -1495,7 +1689,7 @@ class MutableInheritedCompositesTest(_CompositeTestBase, fixtures.MappedTest):
         eq_(f1.data, Point(3, 5))
 
     def test_pickle_of_parent_subclass(self):
-        sess = Session()
+        sess = fixture_session()
         d = Point(3, 4)
         f1 = SubFoo(data=d)
         sess.add(f1)
@@ -1506,7 +1700,7 @@ class MutableInheritedCompositesTest(_CompositeTestBase, fixtures.MappedTest):
         sess.close()
 
         for loads, dumps in picklers():
-            sess = Session()
+            sess = fixture_session()
             f2 = loads(dumps(f1))
             sess.add(f2)
             f2.data.y = 12
