@@ -12,24 +12,26 @@ from sqlalchemy import Integer
 from sqlalchemy import String
 from sqlalchemy import testing
 from sqlalchemy import TypeDecorator
-from sqlalchemy.orm import create_session
-from sqlalchemy.orm import mapper
+from sqlalchemy.orm import make_transient
 from sqlalchemy.orm import relationship
-from sqlalchemy.orm import Session
-from sqlalchemy.orm.session import make_transient
+from sqlalchemy.orm import with_parent
 from sqlalchemy.testing import assert_raises
 from sqlalchemy.testing import assert_raises_message
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import expect_warnings
 from sqlalchemy.testing import fixtures
 from sqlalchemy.testing import ne_
+from sqlalchemy.testing.fixtures import fixture_session
 from sqlalchemy.testing.schema import Column
 from sqlalchemy.testing.schema import Table
 from test.orm import _fixtures
 
 
 def _backend_specific_fk_args():
-    if testing.requires.deferrable_fks.enabled:
+    if (
+        testing.requires.deferrable_fks.enabled
+        and testing.requires.non_updating_cascade.enabled
+    ):
         fk_args = dict(deferrable=True, initially="deferred")
     elif not testing.requires.on_update_cascade.enabled:
         fk_args = dict()
@@ -106,42 +108,44 @@ class NaturalPKTest(fixtures.MappedTest):
     def test_entity(self):
         users, User = self.tables.users, self.classes.User
 
-        mapper(User, users)
+        self.mapper_registry.map_imperatively(User, users)
 
-        sess = create_session()
+        sess = fixture_session()
         u1 = User(username="jack", fullname="jack")
 
         sess.add(u1)
         sess.flush()
-        assert sess.query(User).get("jack") is u1
+        assert sess.get(User, "jack") is u1
 
         u1.username = "ed"
         sess.flush()
 
         def go():
-            assert sess.query(User).get("ed") is u1
+            assert sess.get(User, "ed") is u1
 
         self.assert_sql_count(testing.db, go, 0)
 
-        assert sess.query(User).get("jack") is None
+        assert sess.get(User, "jack") is None
 
         sess.expunge_all()
-        u1 = sess.query(User).get("ed")
+        u1 = sess.get(User, "ed")
         eq_(User(username="ed", fullname="jack"), u1)
 
     def test_load_after_expire(self):
         users, User = self.tables.users, self.classes.User
 
-        mapper(User, users)
+        self.mapper_registry.map_imperatively(User, users)
 
-        sess = create_session()
+        sess = fixture_session()
         u1 = User(username="jack", fullname="jack")
 
         sess.add(u1)
         sess.flush()
-        assert sess.query(User).get("jack") is u1
+        assert sess.get(User, "jack") is u1
 
-        users.update(values={User.username: "jack"}).execute(username="ed")
+        sess.execute(
+            users.update().values({User.username: "jack"}), dict(username="ed")
+        )
 
         # expire/refresh works off of primary key.  the PK is gone
         # in this case so there's no way to look it up.  criterion-
@@ -150,16 +154,16 @@ class NaturalPKTest(fixtures.MappedTest):
         assert_raises(sa.orm.exc.ObjectDeletedError, getattr, u1, "username")
 
         sess.expunge_all()
-        assert sess.query(User).get("jack") is None
-        assert sess.query(User).get("ed").fullname == "jack"
+        assert sess.get(User, "jack") is None
+        assert sess.get(User, "ed").fullname == "jack"
 
     @testing.requires.returning
     def test_update_to_sql_expr(self):
         users, User = self.tables.users, self.classes.User
 
-        mapper(User, users)
+        self.mapper_registry.map_imperatively(User, users)
 
-        sess = create_session()
+        sess = fixture_session()
         u1 = User(username="jack", fullname="jack")
 
         sess.add(u1)
@@ -176,9 +180,9 @@ class NaturalPKTest(fixtures.MappedTest):
         # such as to bump a server side trigger
         users, User = self.tables.users, self.classes.User
 
-        mapper(User, users)
+        self.mapper_registry.map_imperatively(User, users)
 
-        sess = create_session()
+        sess = fixture_session()
         u1 = User(username="jack", fullname="jack")
 
         sess.add(u1)
@@ -193,19 +197,19 @@ class NaturalPKTest(fixtures.MappedTest):
     def test_flush_new_pk_after_expire(self):
         User, users = self.classes.User, self.tables.users
 
-        mapper(User, users)
-        sess = create_session()
+        self.mapper_registry.map_imperatively(User, users)
+        sess = fixture_session()
         u1 = User(username="jack", fullname="jack")
 
         sess.add(u1)
         sess.flush()
-        assert sess.query(User).get("jack") is u1
+        assert sess.get(User, "jack") is u1
 
         sess.expire(u1)
         u1.username = "ed"
         sess.flush()
         sess.expunge_all()
-        assert sess.query(User).get("ed").fullname == "jack"
+        assert sess.get(User, "ed").fullname == "jack"
 
     @testing.requires.on_update_cascade
     def test_onetomany_passive(self):
@@ -222,7 +226,7 @@ class NaturalPKTest(fixtures.MappedTest):
             self.classes.User,
         )
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             User,
             users,
             properties={
@@ -231,16 +235,16 @@ class NaturalPKTest(fixtures.MappedTest):
                 )
             },
         )
-        mapper(Address, addresses)
+        self.mapper_registry.map_imperatively(Address, addresses)
 
-        sess = create_session()
+        sess = fixture_session()
         u1 = User(username="jack", fullname="jack")
         u1.addresses.append(Address(email="jack1"))
         u1.addresses.append(Address(email="jack2"))
         sess.add(u1)
         sess.flush()
 
-        assert sess.query(Address).get("jack1") is u1.addresses[0]
+        assert sess.get(Address, "jack1") is u1.addresses[0]
 
         u1.username = "ed"
         sess.flush()
@@ -252,7 +256,7 @@ class NaturalPKTest(fixtures.MappedTest):
             sess.query(Address).all(),
         )
 
-        u1 = sess.query(User).get("ed")
+        u1 = sess.get(User, "ed")
         u1.username = "jack"
 
         def go():
@@ -271,16 +275,16 @@ class NaturalPKTest(fixtures.MappedTest):
                 username="jack",
                 addresses=[Address(username="jack"), Address(username="jack")],
             )
-            == sess.query(User).get("jack")
+            == sess.get(User, "jack")
         )
 
-        u1 = sess.query(User).get("jack")
+        u1 = sess.get(User, "jack")
         u1.addresses = []
         u1.username = "fred"
         sess.flush()
         sess.expunge_all()
-        assert sess.query(Address).get("jack1").username is None
-        u1 = sess.query(User).get("fred")
+        assert sess.get(Address, "jack1").username is None
+        u1 = sess.get(User, "fred")
         eq_(User(username="fred", fullname="jack"), u1)
 
     @testing.requires.on_update_cascade
@@ -311,18 +315,24 @@ class NaturalPKTest(fixtures.MappedTest):
         )
 
         with testing.db.begin() as conn:
-            conn.execute(users.insert(), username="jack", fullname="jack")
-            conn.execute(addresses.insert(), email="jack1", username="jack")
-            conn.execute(addresses.insert(), email="jack2", username="jack")
+            conn.execute(
+                users.insert(), dict(username="jack", fullname="jack")
+            )
+            conn.execute(
+                addresses.insert(), dict(email="jack1", username="jack")
+            )
+            conn.execute(
+                addresses.insert(), dict(email="jack2", username="jack")
+            )
 
-        mapper(User, users)
-        mapper(
+        self.mapper_registry.map_imperatively(User, users)
+        self.mapper_registry.map_imperatively(
             Address,
             addresses,
             properties={"user": relationship(User, passive_updates=False)},
         )
 
-        sess = create_session()
+        sess = fixture_session()
         u1 = sess.query(User).first()
         a1, a2 = sess.query(Address).all()
         u1.username = "ed"
@@ -340,8 +350,8 @@ class NaturalPKTest(fixtures.MappedTest):
             self.classes.User,
         )
 
-        mapper(User, users)
-        mapper(
+        self.mapper_registry.map_imperatively(User, users)
+        self.mapper_registry.map_imperatively(
             Address,
             addresses,
             properties={
@@ -351,7 +361,7 @@ class NaturalPKTest(fixtures.MappedTest):
             },
         )
 
-        sess = create_session()
+        sess = fixture_session()
         a1 = Address(email="jack1")
         a2 = Address(email="jack2")
         a3 = Address(email="fred")
@@ -419,7 +429,7 @@ class NaturalPKTest(fixtures.MappedTest):
             self.classes.User,
         )
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             User,
             users,
             properties={
@@ -428,9 +438,9 @@ class NaturalPKTest(fixtures.MappedTest):
                 )
             },
         )
-        mapper(Address, addresses)
+        self.mapper_registry.map_imperatively(Address, addresses)
 
-        sess = create_session()
+        sess = fixture_session()
         u1 = User(username="jack", fullname="jack")
         sess.add(u1)
         sess.flush()
@@ -474,8 +484,8 @@ class NaturalPKTest(fixtures.MappedTest):
             self.classes.User,
         )
 
-        mapper(User, users)
-        mapper(
+        self.mapper_registry.map_imperatively(User, users)
+        self.mapper_registry.map_imperatively(
             Address,
             addresses,
             properties={
@@ -485,7 +495,7 @@ class NaturalPKTest(fixtures.MappedTest):
             },
         )
 
-        sess = create_session()
+        sess = fixture_session(autoflush=False)
         a1 = Address(email="jack1")
         a2 = Address(email="jack2")
 
@@ -515,7 +525,7 @@ class NaturalPKTest(fixtures.MappedTest):
             sess.query(Address).all(),
         )
 
-        u1 = sess.query(User).get("ed")
+        u1 = sess.get(User, "ed")
         assert len(u1.addresses) == 2  # load addresses
         u1.username = "fred"
 
@@ -538,7 +548,10 @@ class NaturalPKTest(fixtures.MappedTest):
     def test_manytomany_passive(self):
         self._test_manytomany(True)
 
-    @testing.requires.non_updating_cascade
+    @testing.fails_if(
+        testing.requires.on_update_cascade
+        + testing.requires.sane_multi_rowcount
+    )
     def test_manytomany_nonpassive(self):
         self._test_manytomany(False)
 
@@ -551,7 +564,7 @@ class NaturalPKTest(fixtures.MappedTest):
             self.tables.users_to_items,
         )
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             User,
             users,
             properties={
@@ -563,9 +576,9 @@ class NaturalPKTest(fixtures.MappedTest):
                 )
             },
         )
-        mapper(Item, items)
+        self.mapper_registry.map_imperatively(Item, items)
 
-        sess = create_session()
+        sess = fixture_session()
         u1 = User(username="jack")
         u2 = User(username="fred")
         i1 = Item(itemname="item1")
@@ -606,10 +619,10 @@ class NaturalPKTest(fixtures.MappedTest):
         eq_(["ed", "jack"], sorted([u.username for u in r[1].users]))
 
         sess.expunge_all()
-        u2 = sess.query(User).get(u2.username)
+        u2 = sess.get(User, u2.username)
         u2.username = "wendy"
         sess.flush()
-        r = sess.query(Item).with_parent(u2).all()
+        r = sess.query(Item).filter(with_parent(u2, User.items)).all()
         eq_(Item(itemname="item2"), r[0])
 
     def test_manytoone_deferred_relationship_expr(self):
@@ -622,8 +635,8 @@ class NaturalPKTest(fixtures.MappedTest):
             self.classes.User,
         )
 
-        mapper(User, users)
-        mapper(
+        self.mapper_registry.map_imperatively(User, users)
+        self.mapper_registry.map_imperatively(
             Address,
             addresses,
             properties={
@@ -634,7 +647,7 @@ class NaturalPKTest(fixtures.MappedTest):
             },
         )
 
-        s = Session()
+        s = fixture_session()
         a1 = Address(email="jack1")
         u1 = User(username="jack", fullname="jack")
 
@@ -748,10 +761,12 @@ class TransientExceptionTesst(_fixtures.FixtureTest):
             self.classes.User,
         )
 
-        mapper(User, users)
-        mapper(Address, addresses, properties={"user": relationship(User)})
+        self.mapper_registry.map_imperatively(User, users)
+        self.mapper_registry.map_imperatively(
+            Address, addresses, properties={"user": relationship(User)}
+        )
 
-        sess = create_session()
+        sess = fixture_session()
         u1 = User(id=5, name="u1")
         ad1 = Address(email_address="e1", user=u1)
         sess.add_all([u1, ad1])
@@ -802,9 +817,9 @@ class ReversePKsTest(fixtures.MappedTest):
 
         PUBLISHED, EDITABLE, ARCHIVED = 1, 2, 3
 
-        mapper(User, user)
+        self.mapper_registry.map_imperatively(User, user)
 
-        session = sa.orm.sessionmaker()()
+        session = fixture_session()
 
         a_published = User(1, PUBLISHED, "a")
         session.add(a_published)
@@ -825,16 +840,16 @@ class ReversePKsTest(fixtures.MappedTest):
         a_editable.status = PUBLISHED
 
         session.commit()
-        assert session.query(User).get([1, PUBLISHED]) is a_editable
-        assert session.query(User).get([1, ARCHIVED]) is a_published
+        assert session.get(User, [1, PUBLISHED]) is a_editable
+        assert session.get(User, [1, ARCHIVED]) is a_published
 
         a_published.status = PUBLISHED
         a_editable.status = EDITABLE
 
         session.commit()
 
-        assert session.query(User).get([1, PUBLISHED]) is a_published
-        assert session.query(User).get([1, EDITABLE]) is a_editable
+        assert session.get(User, [1, PUBLISHED]) is a_published
+        assert session.get(User, [1, EDITABLE]) is a_editable
 
     @testing.requires.savepoints
     def test_reverse_savepoint(self):
@@ -842,9 +857,9 @@ class ReversePKsTest(fixtures.MappedTest):
 
         PUBLISHED, EDITABLE, ARCHIVED = 1, 2, 3
 
-        mapper(User, user)
+        self.mapper_registry.map_imperatively(User, user)
 
-        session = sa.orm.sessionmaker()()
+        session = fixture_session()
 
         a_published = User(1, PUBLISHED, "a")
         session.add(a_published)
@@ -871,7 +886,7 @@ class ReversePKsTest(fixtures.MappedTest):
 class SelfReferentialTest(fixtures.MappedTest):
     # mssql, mysql don't allow
     # ON UPDATE on self-referential keys
-    __unsupported_on__ = ("mssql", "mysql")
+    __unsupported_on__ = ("mssql", "mysql", "mariadb")
 
     __requires__ = ("on_update_or_deferrable_fks",)
     __backend__ = True
@@ -896,7 +911,7 @@ class SelfReferentialTest(fixtures.MappedTest):
     def test_one_to_many_on_m2o(self):
         Node, nodes = self.classes.Node, self.tables.nodes
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             Node,
             nodes,
             properties={
@@ -911,7 +926,7 @@ class SelfReferentialTest(fixtures.MappedTest):
             },
         )
 
-        sess = Session()
+        sess = fixture_session(future=True)
         n1 = Node(name="n1")
         sess.add(n1)
         n2 = Node(name="n11", parentnode=n1)
@@ -935,7 +950,7 @@ class SelfReferentialTest(fixtures.MappedTest):
     def test_one_to_many_on_o2m(self):
         Node, nodes = self.classes.Node, self.tables.nodes
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             Node,
             nodes,
             properties={
@@ -949,7 +964,7 @@ class SelfReferentialTest(fixtures.MappedTest):
             },
         )
 
-        sess = Session()
+        sess = fixture_session()
         n1 = Node(name="n1")
         n1.children.append(Node(name="n11"))
         n1.children.append(Node(name="n12"))
@@ -980,7 +995,7 @@ class SelfReferentialTest(fixtures.MappedTest):
     def _test_many_to_one(self, passive):
         Node, nodes = self.classes.Node, self.tables.nodes
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             Node,
             nodes,
             properties={
@@ -990,7 +1005,7 @@ class SelfReferentialTest(fixtures.MappedTest):
             },
         )
 
-        sess = Session()
+        sess = fixture_session()
         n1 = Node(name="n1")
         n11 = Node(name="n11", parentnode=n1)
         n12 = Node(name="n12", parentnode=n1)
@@ -1066,7 +1081,7 @@ class NonPKCascadeTest(fixtures.MappedTest):
             self.tables.addresses,
         )
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             User,
             users,
             properties={
@@ -1075,9 +1090,9 @@ class NonPKCascadeTest(fixtures.MappedTest):
                 )
             },
         )
-        mapper(Address, addresses)
+        self.mapper_registry.map_imperatively(Address, addresses)
 
-        sess = create_session()
+        sess = fixture_session()
         u1 = User(username="jack", fullname="jack")
         u1.addresses.append(Address(email="jack1"))
         u1.addresses.append(Address(email="jack2"))
@@ -1086,17 +1101,17 @@ class NonPKCascadeTest(fixtures.MappedTest):
         a1 = u1.addresses[0]
 
         eq_(
-            sa.select([addresses.c.username]).execute().fetchall(),
+            sess.execute(sa.select(addresses.c.username)).fetchall(),
             [("jack",), ("jack",)],
         )
 
-        assert sess.query(Address).get(a1.id) is u1.addresses[0]
+        assert sess.get(Address, a1.id) is u1.addresses[0]
 
         u1.username = "ed"
         sess.flush()
         assert u1.addresses[0].username == "ed"
         eq_(
-            sa.select([addresses.c.username]).execute().fetchall(),
+            sess.execute(sa.select(addresses.c.username)).fetchall(),
             [("ed",), ("ed",)],
         )
 
@@ -1106,7 +1121,7 @@ class NonPKCascadeTest(fixtures.MappedTest):
             sess.query(Address).all(),
         )
 
-        u1 = sess.query(User).get(u1.id)
+        u1 = sess.get(User, u1.id)
         u1.username = "jack"
 
         def go():
@@ -1125,24 +1140,24 @@ class NonPKCascadeTest(fixtures.MappedTest):
                 username="jack",
                 addresses=[Address(username="jack"), Address(username="jack")],
             )
-            == sess.query(User).get(u1.id)
+            == sess.get(User, u1.id)
         )
         sess.expunge_all()
 
-        u1 = sess.query(User).get(u1.id)
+        u1 = sess.get(User, u1.id)
         u1.addresses = []
         u1.username = "fred"
         sess.flush()
         sess.expunge_all()
-        a1 = sess.query(Address).get(a1.id)
+        a1 = sess.get(Address, a1.id)
         eq_(a1.username, None)
 
         eq_(
-            sa.select([addresses.c.username]).execute().fetchall(),
+            sess.execute(sa.select(addresses.c.username)).fetchall(),
             [(None,), (None,)],
         )
 
-        u1 = sess.query(User).get(u1.id)
+        u1 = sess.get(User, u1.id)
         eq_(User(username="fred", fullname="jack"), u1)
 
 
@@ -1214,7 +1229,7 @@ class CascadeToFKPKTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
             self.tables.addresses,
         )
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             User,
             users,
             properties={
@@ -1223,9 +1238,9 @@ class CascadeToFKPKTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
                 )
             },
         )
-        mapper(Address, addresses)
+        self.mapper_registry.map_imperatively(Address, addresses)
 
-        sess = create_session()
+        sess = fixture_session()
         a1 = Address(username="ed", email="ed@host1")
         u1 = User(username="ed", addresses=[a1])
         u2 = User(username="jack")
@@ -1255,7 +1270,7 @@ class CascadeToFKPKTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
             self.tables.addresses,
         )
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             User,
             users,
             properties={
@@ -1264,9 +1279,9 @@ class CascadeToFKPKTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
                 )
             },
         )
-        mapper(Address, addresses)
+        self.mapper_registry.map_imperatively(Address, addresses)
 
-        sess = create_session()
+        sess = fixture_session(autoflush=False)
         a1 = Address(username="ed", email="ed@host1")
         u1 = User(username="ed", addresses=[a1])
         u2 = User(username="jack")
@@ -1302,8 +1317,8 @@ class CascadeToFKPKTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
             self.tables.addresses,
         )
 
-        mapper(User, users)
-        mapper(
+        self.mapper_registry.map_imperatively(User, users)
+        self.mapper_registry.map_imperatively(
             Address,
             addresses,
             properties={
@@ -1313,7 +1328,7 @@ class CascadeToFKPKTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
             },
         )
 
-        sess = create_session()
+        sess = fixture_session()
         u1 = User(username="jack")
         if uselist:
             a1 = Address(user=[u1], email="foo@bar")
@@ -1344,8 +1359,8 @@ class CascadeToFKPKTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
         )
 
         # tests [ticket:1856]
-        mapper(User, users)
-        mapper(
+        self.mapper_registry.map_imperatively(User, users)
+        self.mapper_registry.map_imperatively(
             Address,
             addresses,
             properties={
@@ -1353,7 +1368,7 @@ class CascadeToFKPKTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
             },
         )
 
-        sess = create_session()
+        sess = fixture_session()
         u1 = User(username="jack")
         u2 = User(username="ed")
         a1 = Address(user=u1, email="foo@bar")
@@ -1371,14 +1386,14 @@ class CascadeToFKPKTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
             self.tables.addresses,
         )
 
-        mapper(User, users)
-        mapper(
+        self.mapper_registry.map_imperatively(User, users)
+        self.mapper_registry.map_imperatively(
             Address,
             addresses,
             properties={"user": relationship(User, passive_updates=True)},
         )
 
-        sess = create_session()
+        sess = fixture_session()
         u1 = User(username="ed")
         a1 = Address(user=u1, email="ed@host1")
 
@@ -1429,7 +1444,7 @@ class CascadeToFKPKTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
             self.tables.addresses,
         )
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             User,
             users,
             properties={
@@ -1438,9 +1453,9 @@ class CascadeToFKPKTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
                 )
             },
         )
-        mapper(Address, addresses)
+        self.mapper_registry.map_imperatively(Address, addresses)
 
-        sess = create_session()
+        sess = fixture_session()
         a1, a2 = (
             Address(username="ed", email="ed@host1"),
             Address(username="ed", email="ed@host2"),
@@ -1451,7 +1466,7 @@ class CascadeToFKPKTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
         eq_(a1.username, "ed")
         eq_(a2.username, "ed")
         eq_(
-            sa.select([addresses.c.username]).execute().fetchall(),
+            sess.execute(sa.select(addresses.c.username)).fetchall(),
             [("ed",), ("ed",)],
         )
 
@@ -1462,7 +1477,7 @@ class CascadeToFKPKTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
         eq_(a1.username, "jack")
         eq_(a2.username, "jack")
         eq_(
-            sa.select([addresses.c.username]).execute().fetchall(),
+            sess.execute(sa.select(addresses.c.username)).fetchall(),
             [("jack",), ("jack",)],
         )
 
@@ -1552,7 +1567,7 @@ class JoinedInheritanceTest(fixtures.MappedTest):
             "person", "manager", "engineer", "owner"
         )
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             Person,
             person,
             polymorphic_on=person.c.type,
@@ -1560,7 +1575,7 @@ class JoinedInheritanceTest(fixtures.MappedTest):
             passive_updates=passive_updates,
         )
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             Engineer,
             engineer,
             inherits=Person,
@@ -1574,12 +1589,12 @@ class JoinedInheritanceTest(fixtures.MappedTest):
             },
         )
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             Manager, manager, inherits=Person, polymorphic_identity="manager"
         )
 
         if threelevel:
-            mapper(
+            self.mapper_registry.map_imperatively(
                 Owner, owner, inherits=Manager, polymorphic_identity="owner"
             )
 
@@ -1620,7 +1635,7 @@ class JoinedInheritanceTest(fixtures.MappedTest):
     def _test_pk(self, passive_updates):
         (Engineer,) = self.classes("Engineer")
         self._mapping_fixture(False, passive_updates)
-        sess = sa.orm.sessionmaker()()
+        sess = fixture_session()
 
         e1 = Engineer(name="dilbert", primary_language="java")
         sess.add(e1)
@@ -1650,7 +1665,7 @@ class JoinedInheritanceTest(fixtures.MappedTest):
 
         self._mapping_fixture(False, passive_updates)
 
-        sess = sa.orm.sessionmaker()()
+        sess = fixture_session()
 
         m1 = Manager(name="dogbert", paperwork="lots")
         e1, e2 = (
@@ -1695,7 +1710,7 @@ class JoinedInheritanceTest(fixtures.MappedTest):
 
         self._mapping_fixture(True, passive_updates)
 
-        sess = sa.orm.sessionmaker()()
+        sess = fixture_session()
 
         o1 = Owner(name="dogbert", owner_name="dog")
         sess.add(o1)
@@ -1733,7 +1748,7 @@ class JoinedInheritanceTest(fixtures.MappedTest):
         Owner, Engineer = self.classes("Owner", "Engineer")
         self._mapping_fixture(True, passive_updates)
 
-        sess = sa.orm.sessionmaker()()
+        sess = fixture_session()
 
         m1 = Owner(name="dogbert", paperwork="lots", owner_name="dog")
         e1, e2 = (
@@ -1777,6 +1792,7 @@ class UnsortablePKTest(fixtures.MappedTest):
     def define_tables(cls, metadata):
         class MyUnsortable(TypeDecorator):
             impl = String(10)
+            cache_ok = True
 
             def process_bind_param(self, value, dialect):
                 return "%s,%s" % (value["x"], value["y"])
@@ -1802,11 +1818,11 @@ class UnsortablePKTest(fixtures.MappedTest):
 
     @classmethod
     def setup_mappers(cls):
-        mapper(cls.classes.Data, cls.tables.data)
+        cls.mapper_registry.map_imperatively(cls.classes.Data, cls.tables.data)
 
     def test_updates_sorted(self):
         Data = self.classes.Data
-        s = Session()
+        s = fixture_session()
 
         s.add_all(
             [
@@ -1884,21 +1900,21 @@ class JoinedInheritancePKOnFKTest(fixtures.MappedTest):
             self.tables.engineer,
         )
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             Person,
             person,
             polymorphic_on=person.c.type,
             polymorphic_identity="person",
             passive_updates=passive_updates,
         )
-        mapper(
+        self.mapper_registry.map_imperatively(
             Engineer,
             engineer,
             inherits=Person,
             polymorphic_identity="engineer",
         )
 
-        sess = sa.orm.sessionmaker()()
+        sess = fixture_session()
 
         e1 = Engineer(name="dilbert", primary_language="java")
         sess.add(e1)
