@@ -16,6 +16,7 @@ from sqlalchemy import String
 from sqlalchemy import testing
 from sqlalchemy.orm import aliased
 from sqlalchemy.orm import defer
+from sqlalchemy.orm import join as orm_join
 from sqlalchemy.orm import joinedload
 from sqlalchemy.orm import lazyload
 from sqlalchemy.orm import registry
@@ -26,6 +27,7 @@ from sqlalchemy.orm import subqueryload
 from sqlalchemy.orm import with_loader_criteria
 from sqlalchemy.orm.decl_api import declared_attr
 from sqlalchemy.testing import eq_
+from sqlalchemy.testing import expect_raises_message
 from sqlalchemy.testing.assertions import expect_raises
 from sqlalchemy.testing.assertsql import CompiledSQL
 from sqlalchemy.testing.fixtures import fixture_session
@@ -55,6 +57,33 @@ class _Fixtures(_fixtures.FixtureTest):
         )
         return User, Address
 
+    @testing.fixture
+    def user_address_custom_strat_fixture(self):
+        users, Address, addresses, User = (
+            self.tables.users,
+            self.classes.Address,
+            self.tables.addresses,
+            self.classes.User,
+        )
+
+        def go(strat):
+            self.mapper_registry.map_imperatively(
+                User,
+                users,
+                properties={
+                    "addresses": relationship(
+                        self.mapper_registry.map_imperatively(
+                            Address, addresses
+                        ),
+                        lazy=strat,
+                        order_by=Address.id,
+                    )
+                },
+            )
+            return User, Address
+
+        return go
+
     @testing.fixture
     def order_item_fixture(self):
         Order, Item = self.classes("Order", "Item")
@@ -204,6 +233,22 @@ class LoaderCriteriaTest(_Fixtures, testing.AssertsCompiledSQL):
             "FROM users WHERE users.name != :name_1",
         )
 
+    def test_err_given_in_pathed(self, user_address_fixture):
+        User, Address = user_address_fixture
+
+        with expect_raises_message(
+            sa_exc.ArgumentError,
+            r"Loader option <.*LoaderCriteriaOption.*> is not compatible "
+            r"with the Load.options\(\) method.",
+        ):
+            select(User).options(
+                selectinload(User.addresses).options(
+                    with_loader_criteria(
+                        Address, Address.email_address != "foo"
+                    )
+                )
+            )
+
     def test_criteria_post_replace(self, user_address_fixture):
         User, Address = user_address_fixture
 
@@ -220,6 +265,178 @@ class LoaderCriteriaTest(_Fixtures, testing.AssertsCompiledSQL):
             "WHERE users.name != :name_1",
         )
 
+    @testing.combinations(
+        (
+            lambda User, Address: select(Address)
+            .select_from(User)
+            .join(User.addresses)
+            .options(with_loader_criteria(User, User.name != "name")),
+        ),
+        (
+            lambda User, Address: select(Address)
+            .select_from(orm_join(User, Address, User.addresses))
+            .options(with_loader_criteria(User, User.name != "name")),
+        ),
+        (
+            lambda User, Address: select(Address)
+            .join_from(User, Address, User.addresses)
+            .options(with_loader_criteria(User, User.name != "name")),
+        ),
+        argnames="stmt_fn",
+    )
+    @testing.combinations(True, False, argnames="alias_user")
+    def test_criteria_select_from_w_join_left(
+        self, user_address_fixture, stmt_fn, alias_user
+    ):
+        """test #8721"""
+        User, Address = user_address_fixture
+
+        if alias_user:
+            User = aliased(User)
+
+        stmt = testing.resolve_lambda(stmt_fn, User=User, Address=Address)
+
+        if alias_user:
+            self.assert_compile(
+                stmt,
+                "SELECT addresses.id, addresses.user_id, "
+                "addresses.email_address FROM users AS users_1 "
+                "JOIN addresses ON users_1.id = addresses.user_id "
+                "WHERE users_1.name != :name_1",
+            )
+        else:
+            self.assert_compile(
+                stmt,
+                "SELECT addresses.id, addresses.user_id, "
+                "addresses.email_address "
+                "FROM users JOIN addresses ON users.id = addresses.user_id "
+                "WHERE users.name != :name_1",
+            )
+
+    @testing.combinations(
+        (
+            lambda User, Address: select(Address.id, User.id)
+            .select_from(User)
+            .join(User.addresses)
+            .options(with_loader_criteria(User, User.name != "name")),
+        ),
+        (
+            lambda User, Address: select(Address.id, User.id)
+            .select_from(orm_join(User, Address, User.addresses))
+            .options(with_loader_criteria(User, User.name != "name")),
+        ),
+        (
+            lambda User, Address: select(Address.id, User.id)
+            .join_from(User, Address, User.addresses)
+            .options(with_loader_criteria(User, User.name != "name")),
+        ),
+        argnames="stmt_fn",
+    )
+    @testing.combinations(True, False, argnames="alias_user")
+    def test_criteria_select_from_w_join_left_including_entity(
+        self, user_address_fixture, stmt_fn, alias_user
+    ):
+        """test #8721"""
+        User, Address = user_address_fixture
+
+        if alias_user:
+            User = aliased(User)
+
+        stmt = testing.resolve_lambda(stmt_fn, User=User, Address=Address)
+
+        if alias_user:
+            self.assert_compile(
+                stmt,
+                "SELECT addresses.id, users_1.id AS id_1 "
+                "FROM users AS users_1 JOIN addresses "
+                "ON users_1.id = addresses.user_id "
+                "WHERE users_1.name != :name_1",
+            )
+        else:
+            self.assert_compile(
+                stmt,
+                "SELECT addresses.id, users.id AS id_1 "
+                "FROM users JOIN addresses ON users.id = addresses.user_id "
+                "WHERE users.name != :name_1",
+            )
+
+    @testing.combinations(
+        (
+            lambda User, Address: select(Address)
+            .select_from(User)
+            .join(User.addresses)
+            .options(
+                with_loader_criteria(Address, Address.email_address != "email")
+            ),
+        ),
+        (
+            # for orm_join(), this is set up before we have the context
+            # available that allows with_loader_criteria to be set up
+            # correctly
+            lambda User, Address: select(Address)
+            .select_from(orm_join(User, Address, User.addresses))
+            .options(
+                with_loader_criteria(Address, Address.email_address != "email")
+            ),
+            testing.fails("not implemented right now"),
+        ),
+        (
+            lambda User, Address: select(Address)
+            .join_from(User, Address, User.addresses)
+            .options(
+                with_loader_criteria(Address, Address.email_address != "email")
+            ),
+        ),
+        argnames="stmt_fn",
+    )
+    def test_criteria_select_from_w_join_right(
+        self, user_address_fixture, stmt_fn
+    ):
+        """test #8721"""
+        User, Address = user_address_fixture
+
+        stmt = testing.resolve_lambda(stmt_fn, User=User, Address=Address)
+        self.assert_compile(
+            stmt,
+            "SELECT addresses.id, addresses.user_id, addresses.email_address "
+            "FROM users JOIN addresses ON users.id = addresses.user_id "
+            "AND addresses.email_address != :email_address_1",
+        )
+
+    @testing.combinations(
+        "select",
+        "joined",
+        "subquery",
+        "selectin",
+        "immediate",
+        argnames="loader_strategy",
+    )
+    def test_loader_strategy_on_refresh(
+        self, loader_strategy, user_address_custom_strat_fixture
+    ):
+        User, Address = user_address_custom_strat_fixture(loader_strategy)
+
+        sess = fixture_session()
+
+        @event.listens_for(sess, "do_orm_execute")
+        def add_criteria(orm_context):
+            orm_context.statement = orm_context.statement.options(
+                with_loader_criteria(
+                    Address,
+                    ~Address.id.in_([5, 3]),
+                )
+            )
+
+        u1 = sess.get(User, 7)
+        u2 = sess.get(User, 8)
+        eq_(u1.addresses, [Address(id=1)])
+        eq_(u2.addresses, [Address(id=2), Address(id=4)])
+
+        for i in range(3):
+            sess.expire_all()
+            eq_(u1.addresses, [Address(id=1)])
+            eq_(u2.addresses, [Address(id=2), Address(id=4)])
+
     def test_criteria_post_replace_legacy(self, user_address_fixture):
         User, Address = user_address_fixture
 
@@ -420,6 +637,63 @@ class LoaderCriteriaTest(_Fixtures, testing.AssertsCompiledSQL):
             ),
         )
 
+    def test_select_selectinload_mapper_mapper_closure_criteria(
+        self, user_address_fixture
+    ):
+        User, Address = user_address_fixture
+
+        def get_statement(closure="name"):
+
+            stmt = select(User).options(
+                selectinload(User.addresses),
+                with_loader_criteria(
+                    Address, lambda cls: cls.email_address != closure
+                ),
+            )
+            return stmt
+
+        s = Session(testing.db, future=True)
+
+        stmt = get_statement(closure="name")
+        with self.sql_execution_asserter() as asserter:
+            s.execute(stmt).all()
+
+        asserter.assert_(
+            CompiledSQL(
+                "SELECT users.id, users.name FROM users",
+                [],
+            ),
+            CompiledSQL(
+                "SELECT addresses.user_id AS addresses_user_id, addresses.id "
+                "AS addresses_id, addresses.email_address "
+                "AS addresses_email_address FROM addresses "
+                "WHERE addresses.user_id IN (__[POSTCOMPILE_primary_keys]) "
+                "AND addresses.email_address != :closure_1 "
+                "ORDER BY addresses.id",
+                [{"primary_keys": [7, 8, 9, 10], "closure_1": "name"}],
+            ),
+        )
+
+        stmt = get_statement(closure="new name")
+        with self.sql_execution_asserter() as asserter:
+            s.execute(stmt).all()
+
+        asserter.assert_(
+            CompiledSQL(
+                "SELECT users.id, users.name FROM users",
+                [],
+            ),
+            CompiledSQL(
+                "SELECT addresses.user_id AS addresses_user_id, addresses.id "
+                "AS addresses_id, addresses.email_address "
+                "AS addresses_email_address FROM addresses "
+                "WHERE addresses.user_id IN (__[POSTCOMPILE_primary_keys]) "
+                "AND addresses.email_address != :closure_1 "
+                "ORDER BY addresses.id",
+                [{"primary_keys": [7, 8, 9, 10], "closure_1": "new name"}],
+            ),
+        )
+
     def test_select_lazyload_mapper_mapper_criteria(
         self, user_address_fixture
     ):
@@ -482,6 +756,125 @@ class LoaderCriteriaTest(_Fixtures, testing.AssertsCompiledSQL):
             ),
         )
 
+    def test_select_lazyload_mapper_mapper_closure_criteria(
+        self, user_address_fixture
+    ):
+        User, Address = user_address_fixture
+
+        def get_statement(closure="name"):
+
+            stmt = (
+                select(User)
+                .options(
+                    lazyload(User.addresses),
+                    with_loader_criteria(
+                        Address, lambda cls: cls.email_address != closure
+                    ),
+                )
+                .order_by(User.id)
+            )
+            return stmt
+
+        s = Session(testing.db, future=True)
+
+        stmt = get_statement(closure="name")
+        with self.sql_execution_asserter() as asserter:
+            for obj in s.scalars(stmt).all():
+                obj.addresses
+
+        asserter.assert_(
+            CompiledSQL(
+                "SELECT users.id, users.name FROM users ORDER BY users.id",
+                [],
+            ),
+            CompiledSQL(
+                "SELECT addresses.id AS addresses_id, "
+                "addresses.user_id AS addresses_user_id, "
+                "addresses.email_address AS addresses_email_address "
+                "FROM addresses WHERE :param_1 = addresses.user_id "
+                "AND addresses.email_address != :closure_1 "
+                "ORDER BY addresses.id",
+                [{"param_1": 7, "closure_1": "name"}],
+            ),
+            CompiledSQL(
+                "SELECT addresses.id AS addresses_id, "
+                "addresses.user_id AS addresses_user_id, "
+                "addresses.email_address AS addresses_email_address "
+                "FROM addresses WHERE :param_1 = addresses.user_id "
+                "AND addresses.email_address != :closure_1 "
+                "ORDER BY addresses.id",
+                [{"param_1": 8, "closure_1": "name"}],
+            ),
+            CompiledSQL(
+                "SELECT addresses.id AS addresses_id, "
+                "addresses.user_id AS addresses_user_id, "
+                "addresses.email_address AS addresses_email_address "
+                "FROM addresses WHERE :param_1 = addresses.user_id "
+                "AND addresses.email_address != :closure_1 "
+                "ORDER BY addresses.id",
+                [{"param_1": 9, "closure_1": "name"}],
+            ),
+            CompiledSQL(
+                "SELECT addresses.id AS addresses_id, "
+                "addresses.user_id AS addresses_user_id, "
+                "addresses.email_address AS addresses_email_address "
+                "FROM addresses WHERE :param_1 = addresses.user_id "
+                "AND addresses.email_address != :closure_1 "
+                "ORDER BY addresses.id",
+                [{"param_1": 10, "closure_1": "name"}],
+            ),
+        )
+
+        stmt = get_statement(closure="new name")
+        with self.sql_execution_asserter() as asserter:
+            for obj in s.scalars(
+                stmt, execution_options={"populate_existing": True}
+            ).all():
+                obj.addresses
+
+        asserter.assert_(
+            CompiledSQL(
+                "SELECT users.id, users.name FROM users ORDER BY users.id",
+                [],
+            ),
+            CompiledSQL(
+                "SELECT addresses.id AS addresses_id, "
+                "addresses.user_id AS addresses_user_id, "
+                "addresses.email_address AS addresses_email_address "
+                "FROM addresses WHERE :param_1 = addresses.user_id "
+                "AND addresses.email_address != :closure_1 "
+                "ORDER BY addresses.id",
+                [{"param_1": 7, "closure_1": "new name"}],
+            ),
+            CompiledSQL(
+                "SELECT addresses.id AS addresses_id, "
+                "addresses.user_id AS addresses_user_id, "
+                "addresses.email_address AS addresses_email_address "
+                "FROM addresses WHERE :param_1 = addresses.user_id "
+                "AND addresses.email_address != :closure_1 "
+                "ORDER BY addresses.id",
+                [{"param_1": 8, "closure_1": "new name"}],
+            ),
+            CompiledSQL(
+                "SELECT addresses.id AS addresses_id, "
+                "addresses.user_id AS addresses_user_id, "
+                "addresses.email_address AS addresses_email_address "
+                "FROM addresses WHERE :param_1 = addresses.user_id "
+                "AND addresses.email_address != :closure_1 "
+                "ORDER BY addresses.id",
+                [{"param_1": 9, "closure_1": "new name"}],
+            ),
+            CompiledSQL(
+                "SELECT addresses.id AS addresses_id, "
+                "addresses.user_id AS addresses_user_id, "
+                "addresses.email_address AS addresses_email_address "
+                "FROM addresses WHERE :param_1 = addresses.user_id "
+                "AND addresses.email_address != :closure_1 "
+                "ORDER BY addresses.id",
+                [{"param_1": 10, "closure_1": "new name"}],
+            ),
+        )
+
     def test_select_aliased_inclaliased_criteria(self, user_address_fixture):
         User, Address = user_address_fixture
 
