@@ -5,13 +5,13 @@ from sqlalchemy import Integer
 from sqlalchemy import select
 from sqlalchemy import String
 from sqlalchemy import testing
-from sqlalchemy.orm import mapper
 from sqlalchemy.orm import Session
 from sqlalchemy.testing import assert_raises
 from sqlalchemy.testing import assert_raises_message
 from sqlalchemy.testing import AssertsCompiledSQL
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import fixtures
+from sqlalchemy.testing.fixtures import fixture_session
 from sqlalchemy.testing.schema import Column
 from sqlalchemy.testing.schema import Table
 
@@ -40,11 +40,13 @@ class SelectableNoFromsTest(fixtures.MappedTest, AssertsCompiledSQL):
     def test_no_tables(self):
         Subset = self.classes.Subset
 
-        selectable = select([column("x"), column("y"), column("z")]).alias()
-        mapper(Subset, selectable, primary_key=[selectable.c.x])
+        selectable = select(column("x"), column("y"), column("z")).alias()
+        self.mapper_registry.map_imperatively(
+            Subset, selectable, primary_key=[selectable.c.x]
+        )
 
         self.assert_compile(
-            Session().query(Subset),
+            fixture_session().query(Subset),
             "SELECT anon_1.x AS anon_1_x, anon_1.y AS anon_1_y, "
             "anon_1.z AS anon_1_z FROM (SELECT x, y, z) AS anon_1",
             use_default_dialect=True,
@@ -53,11 +55,11 @@ class SelectableNoFromsTest(fixtures.MappedTest, AssertsCompiledSQL):
     def test_no_table_needs_pl(self):
         Subset = self.classes.Subset
 
-        selectable = select([column("x"), column("y"), column("z")]).alias()
+        selectable = select(column("x"), column("y"), column("z")).alias()
         assert_raises_message(
             sa.exc.ArgumentError,
             "could not assemble any primary key columns",
-            mapper,
+            self.mapper_registry.map_imperatively,
             Subset,
             selectable,
         )
@@ -65,16 +67,19 @@ class SelectableNoFromsTest(fixtures.MappedTest, AssertsCompiledSQL):
     def test_no_selects(self):
         Subset, common = self.classes.Subset, self.tables.common
 
-        subset_select = select([common.c.id, common.c.data])
+        subset_select = select(common.c.id, common.c.data)
         assert_raises(
-            sa.exc.InvalidRequestError, mapper, Subset, subset_select
+            sa.exc.ArgumentError,
+            self.mapper_registry.map_imperatively,
+            Subset,
+            subset_select,
         )
 
     def test_basic(self):
         Subset, common = self.classes.Subset, self.tables.common
 
-        subset_select = select([common.c.id, common.c.data]).alias()
-        mapper(Subset, subset_select)
+        subset_select = select(common.c.id, common.c.data).alias()
+        self.mapper_registry.map_imperatively(Subset, subset_select)
         sess = Session(bind=testing.db)
         sess.add(Subset(data=1))
         sess.flush()
